%{
/*


NOTE: Book pgs 418-419:
ralloc frees the input registers before allocating the output register,
which allows it to reuse an input register as the output register. This
econonmy is always safe when the node is implemented by a single instruction,
but it can be unsafe if a node is implemented by a sequence of instructions:
If the output register is also one of the input registers, and if the
sequence changes the output register before reading the corresponding input
register, then the read fetches a corrupt value. We take care that all rules
that emit instruction sequences set their output register ONLY after they
finish reading all input registers. Most templates emit just one instruction,
so this assumption is a good default, but it does require considerable care
with multi-instruction sequences.






stmt: LEI4(addr56,rc)		"cmp %0,%1 ;LE1\nljle =%a\n"  5
stmt: LEU4(addr56,rc)		"cmp %0,%1 ;LE2\nljle =%a\n"  5
stmt: LEI4(reg,a56rc)		"cmp %0,%1 ;LE3\nljle =%a\n"  4
stmt: LEU4(reg,a56rc)		"cmp %0,%1 ;LE4\nljle =%a\n"  4

a56rc:	addr56				"%0"
a56rc:	reg					"%0"
a56rc:	con					"=%0"

Possible assignments?

stmt: ASGNU1(addr56,con)	"ldm %0b,=%1\n"		1

stmt: ASGNU2(addr56,reg1)	"ldm r110,%1b\nlde %0,r110\n"		2
stmt: ASGNU2(addr56,reg2)	"ldm %0w,%1w\n"		1
stmt: ASGNU2(addr56,reg4)	"ldm %0w,%1w\n"		1
stmt: ASGNU2(addr56,reg8)	"ldm %0w,%1w\n"		1

*/
/*
	R13 is a DWORD scratch
	R14, R15 are ASGN+B working registers
	R16 (and R17) are return values
	R22-R29 are DWORD regs
	R51 is a WORD scratch
	R52-R55 is a QWORD/FLOAT scratch
	R56-R59 is a QWORD reg
	R60-R63 is a QWORD reg
	R64-R67 is a QWORD reg
	R68-R71 is a QWORD reg
	R72-R79 are WORD regs
	R80-R87 are BYTE regs
	R88-R95 is a FLOAT reg
	R96-103 is a FLOAT reg
	R110 is a BYTE scratch
*/
enum {	R14=0,   R15=1,   R16=2,   R22=3,  R23=4,  R24=5,  R25=6,  R26=7,
		R27=8,   R28=9,   R29=10,  R56=11, R60=12, R64=13, R68=14, R72=15,
		R73=16,  R74=17,  R75=18,  R76=19, R77=20, R78=21, R79=22, R80=23,
		R81=24,  R82=25,  R83=26,  R84=27, R85=28, R86=29, R87=30};

#include <io.h>
#include <fcntl.h>
#include <ctype.h>
#include <sys/stat.h>

#include "c.h"
#define NODEPTR_TYPE Node
#define OP_LABEL(p) ((p)->op)
#define LEFT_CHILD(p) ((p)->kids[0])
#define RIGHT_CHILD(p) ((p)->kids[1])
#define STATE_LABEL(p) ((p)->x.state)
static void address(Symbol, Symbol, long);
static void blkfetch(long, long, long, long);
static void blkloop(long, long, long, long, long, long[]);
static void blkstore(long, long, long, long);
static void defaddress(Symbol);
static void defconst(long, long, Value);
static void defstring(long, char *);
static void defsymbol(Symbol);
static void doarg(Node);
static void emit2(Node);
static void export(Symbol);
static void clobber(Node);
static void function(Symbol, Symbol [], Symbol [], long);
static void global(Symbol);
static void import(Symbol);
static void local(Symbol);
static void progbeg(long, char **);
static void progend(void);
static void segment(long);
static void space(long);
static void target(Node);
extern long ckstack(Node, long);
long memop56(Node, long);
extern long sametree(Node, Node);
static Symbol charreg[32], shortreg[32], longreg[32], longlongreg[32];
static Symbol fltreg[32];

static Symbol charregw, shortregw, longregw, longlongregw, fltregw, asmresult;

static long cseg, exchkstk;

//static Symbol quo, rem;

%}
%start stmt
%term CNSTF4=4113
%term CNSTF8=8209
%term CNSTI1=1045
%term CNSTI2=2069
%term CNSTI4=4117
%term CNSTI8=8213

%term CNSTP4=4119

%term CNSTU1=1046
%term CNSTU2=2070
%term CNSTU4=4118
%term CNSTU8=8214

%term ARGB=41
%term ARGI2=2085
%term ARGU2=2086
%term ARGF4=4129
%term ARGF8=8225
%term ARGI4=4133
%term ARGI8=8229
%term ARGP4=4135
%term ARGU4=4134
%term ARGU8=8230

%term ASGNB=57
%term ASGNF4=4145
%term ASGNF8=8241
%term ASGNI1=1077
%term ASGNI2=2101
%term ASGNI4=4149
%term ASGNI8=8245
%term ASGNP4=4151
%term ASGNU1=1078
%term ASGNU2=2102
%term ASGNU4=4150
%term ASGNU8=8246

%term INDIRB=73
%term INDIRF4=4161
%term INDIRF8=8257
%term INDIRI1=1093
%term INDIRI2=2117
%term INDIRI4=4165
%term INDIRI8=8261
%term INDIRP4=4167
%term INDIRU1=1094
%term INDIRU2=2118
%term INDIRU4=4166
%term INDIRU8=8262

%term CVFF4=4209
%term CVFF8=8305
%term CVFI2=2165
%term CVFI4=4213
%term CVFI8=8309

%term CVII1=1157
%term CVII2=2181
%term CVII4=4229
%term CVII8=8325
%term CVIU1=1158
%term CVIU2=2182
%term CVIU4=4230
%term CVIU8=8326
%term CVIF4=4225
%term CVIF8=8321

%term CVUI1=1205
%term CVUI2=2229
%term CVUI4=4277
%term CVUI8=8373
%term CVUU1=1206
%term CVUU2=2230
%term CVUU4=4278
%term CVUU8=8374
%term CVUP4=4279
%term CVUF4=4273
%term CVUF8=8369

%term CVPP4=4247
%term CVPU4=4246
%term CVPU8=8342

%term NEGI1=1221
%term NEGI2=2245
%term NEGI4=4293
%term NEGI8=8389
%term NEGF4=4289
%term NEGF8=8385

%term CALLB=217
%term CALLI2=2261
%term CALLU2=2262
%term CALLF4=4305
%term CALLF8=8401
%term CALLI4=4309
%term CALLI8=8405
%term CALLP4=4311
%term CALLU4=4310
%term CALLU8=8406
%term CALLV=216

%term RETI4=4341
%term RETU4=4342
%term RETP4=4343
%term RETI8=8437
%term RETU8=8438
%term RETF4=4337
%term RETF8=8433
%term RETV=248

%term ADDRFP4=4375
%term ADDRGP4=4359
%term ADDRLP4=4391

%term ADDI1=1333
%term ADDU1=1334
%term ADDI2=2357
%term ADDU2=2358
%term ADDI4=4405
%term ADDU4=4406
%term ADDI8=8501
%term ADDU8=8502
%term ADDP4=4407
%term ADDF4=4401
%term ADDF8=8497

%term SUBI1=1349
%term SUBU1=1350
%term SUBI2=2373
%term SUBU2=2374
%term SUBI4=4421
%term SUBU4=4422
%term SUBI8=8517
%term SUBU8=8518
%term SUBP4=4423
%term SUBF4=4417
%term SUBF8=8513

%term DIVI1=1477
%term DIVU1=1478
%term DIVI2=2501
%term DIVU2=2502
%term DIVI4=4549
%term DIVU4=4550
%term DIVI8=8645
%term DIVU8=8646
%term DIVF4=4545
%term DIVF8=8641

%term MULI1=1493
%term MULU1=1494
%term MULI2=2517
%term MULU2=2518
%term MULI4=4565
%term MULU4=4566
%term MULI8=8661
%term MULU8=8662
%term MULF4=4561
%term MULF8=8657

%term LSHI1=1365
%term LSHU1=1366
%term LSHI2=2389
%term LSHU2=2390
%term LSHI4=4437
%term LSHU4=4438
%term LSHI8=8533
%term LSHU8=8534

%term MODI1=1381
%term MODU1=1382
%term MODI2=2405
%term MODU2=2406
%term MODI4=4453
%term MODU4=4454
%term MODI8=8549
%term MODU8=8550

%term RSHI1=1397
%term RSHU1=1398
%term RSHI2=2421
%term RSHU2=2422
%term RSHI4=4469
%term RSHU4=4470
%term RSHI8=8565
%term RSHU8=8566

%term BANDI1=1413
%term BANDU1=1414
%term BANDI2=2437
%term BANDU2=2438
%term BANDI4=4485
%term BANDU4=4486
%term BANDI8=8581
%term BANDU8=8582

%term BCOMI1=1429
%term BCOMU1=1430
%term BCOMI2=2453
%term BCOMU2=2454
%term BCOMI4=4501
%term BCOMU4=4502
%term BCOMI8=8597
%term BCOMU8=8598

%term BORI1=1445
%term BORU1=1446
%term BORI2=2469
%term BORU2=2470
%term BORI4=4517
%term BORU4=4518
%term BORI8=8613
%term BORU8=8614

%term BXORI1=1461
%term BXORU1=1462
%term BXORI2=2485
%term BXORU2=2486
%term BXORI4=4533
%term BXORU4=4534
%term BXORI8=8629
%term BXORU8=8630

%term EQI1=1509
%term EQU1=1510
%term EQI2=2533
%term EQU2=2534
%term EQI4=4581
%term EQU4=4582
%term EQI8=8677
%term EQU8=8678
%term EQF4=4577
%term EQF8=8673

%term GEI1=1525
%term GEU1=1526
%term GEI2=2549
%term GEU2=2550
%term GEI4=4597
%term GEU4=4598
%term GEI8=8693
%term GEU8=8694
%term GEF4=4593
%term GEF8=8689

%term GTI1=1541
%term GTU1=1542
%term GTI2=2565
%term GTU2=2566
%term GTI4=4613
%term GTU4=4614
%term GTI8=8709
%term GTU8=8710
%term GTF4=4609
%term GTF8=8705

%term LEI1=1557
%term LEU1=1558
%term LEI2=2581
%term LEU2=2582
%term LEI4=4629
%term LEU4=4630
%term LEI8=8725
%term LEU8=8726
%term LEF4=4625
%term LEF8=8721

%term LTI1=1573
%term LTU1=1574
%term LTI2=2597
%term LTU2=2598
%term LTI4=4645
%term LTU4=4646
%term LTI8=8741
%term LTU8=8742
%term LTF4=4641
%term LTF8=8737

%term NEI1=1589
%term NEU1=1590
%term NEI2=2613
%term NEU2=2614
%term NEI4=4661
%term NEU4=4662
%term NEI8=8757
%term NEU8=8758
%term NEF4=4657
%term NEF8=8753

%term JUMPV=584

%term LABELV=600

%term ASM0=720
%term ASM1=736
%term ASM2=752
%term ASMRETU4=4870
%term ASMRETU8=8966

%term LOADB=233
%term LOADI1=1253
%term LOADU1=1254
%term LOADI2=2277
%term LOADU2=2278
%term LOADI4=4325
%term LOADU4=4326
%term LOADP4=4327
%term LOADI8=8421
%term LOADU8=8422
%term LOADF4=4321
%term LOADF8=8417

%term VREGP=711
%%

con: CNSTI1	"%a"
con: CNSTU1	"%a"
con: CNSTI2	"%a"
con: CNSTU2	"%a"
con: CNSTI4	"%a"
con: CNSTU4	"%a"
con: CNSTP4	"%a"
con: CNSTI8	"%a"
con: CNSTU8	"%a"

conm1:  CNSTI1  "-1"  range(a, -1, -1)
conm1:  CNSTU1  "-1"  range(a, -1, -1)
conm1:  CNSTI2  "-1"  range(a, -1, -1)
conm1:  CNSTU2  "-1"  range(a, -1, -1)
conm1:  CNSTI4  "-1"  range(a, -1, -1)
conm1:  CNSTU4  "-1"  range(a, -1, -1)

con0:	CNSTI1 "0"	range(a, 0, 0)
con0:	CNSTU1 "0"	range(a, 0, 0)
con0:	CNSTI2 "0"	range(a, 0, 0)
con0:	CNSTU2 "0"	range(a, 0, 0)
con0:	CNSTI4 "0"	range(a, 0, 0)
con0:	CNSTU4 "0"	range(a, 0, 0)

con1:  CNSTI1  "1"  range(a, 1, 1)
con1:  CNSTU1  "1"  range(a, 1, 1)
con1:  CNSTI2  "1"  range(a, 1, 1)
con1:  CNSTU2  "1"  range(a, 1, 1)
con1:  CNSTI4  "1"  range(a, 1, 1)
con1:  CNSTU4  "1"  range(a, 1, 1)

con2:	CNSTI4 "2"	range(a, 2, 2)
con2:	CNSTU4 "2"	range(a, 2, 2)
con4:	CNSTI4 "4"	range(a, 4, 4)
con4:	CNSTU4 "4"	range(a, 4, 4)
con8:	CNSTI4 "8"	range(a, 8, 8)
con8:	CNSTU4 "8"	range(a, 8, 8)
con16:	CNSTI4 "16"	range(a, 16, 16)
con16:	CNSTU4 "16"	range(a, 16, 16)
con32:	CNSTI4 "32"	range(a, 32, 32)
con32:	CNSTU4 "32"	range(a, 32, 32)
con64:	CNSTI4 "64"	range(a, 64, 64)
con64:	CNSTU4 "64"	range(a, 64, 64)
con128:	CNSTI4 "128"	range(a, 128, 128)
con128:	CNSTU4 "128"	range(a, 128, 128)
con256:	CNSTI4 "256"	range(a, 256, 256)
con256:	CNSTU4 "256"	range(a, 256, 256)

reg1: INDIRI1(VREGP)     "# read register\n"
reg1: INDIRU1(VREGP)     "# read register\n"
reg2: INDIRI2(VREGP)     "# read register\n"
reg2: INDIRU2(VREGP)     "# read register\n"
reg4: INDIRI4(VREGP)     "# read register\n"
reg4: INDIRU4(VREGP)     "# read register\n"
reg4: INDIRP4(VREGP)     "# read register\n"
reg4: INDIRF4(VREGP)     "# read register\n"
reg8: INDIRI8(VREGP)     "# read register\n"
reg8: INDIRU8(VREGP)     "# read register\n"
reg8: INDIRF8(VREGP)     "# read register\n"

reg1: INDIRI1(addr56)	"ldm %cb,%0\n"	2
reg1: INDIRU1(addr56)	"ldm %cb,%0\n"	2
reg2: INDIRI2(addr56)	"ldm %cw,%0\n"	2
reg2: INDIRU2(addr56)	"ldm %cw,%0\n"	2
reg4: INDIRI4(addr56)	"ldm %cd,%0\n"	2
reg4: INDIRU4(addr56)	"ldm %cd,%0\n"	2
reg4: INDIRP4(addr56)	"ldm %cd,%0\n"	2
reg8: INDIRI8(addr56)	"ldm %cq,%0\n"	2
reg8: INDIRU8(addr56)	"ldm %cq,%0\n"	2

reg1: LOADI1(reg1)		"# move\n"  1
reg1: LOADU1(reg1)		"# move\n"  1
reg1: LOADI1(reg2)		"# move\n"  1
reg1: LOADU1(reg2)		"# move\n"  1
reg1: LOADI1(reg4)		"# move\n"  1
reg1: LOADU1(reg4)		"# move\n"	1
reg1: LOADI1(reg8)		"# move\n"  1
reg1: LOADU1(reg8)		"# move\n"  1
reg1: LOADU4(indirmem51)	"ldm %ab,%0b\n"			1

reg2: LOADI2(reg2)		"# move\n"  1
reg2: LOADU2(reg2)		"# move\n"  1
reg2: LOADI2(reg4)		"# move\n"  1
reg2: LOADU2(reg4)		"# move\n"  1
reg2: LOADI2(reg8)		"# move\n"  1
reg2: LOADU2(reg8)		"# move\n"  1
reg2: LOADU4(indirmem52)	"ldm %aw,%0w\n"			1

reg4: LOADI4(reg4)		"# move\n"  1
reg4: LOADU4(reg4)		"# move\n"  1
reg4: LOADI4(reg8)		"# move\n"  1
reg4: LOADU4(reg8)		"# move\n"  1
reg4: LOADP4(reg4)		"# move\n"  1
reg4: LOADU4(indirmem54)	"ldm %cd%,%0d\n"			1

reg8: LOADI8(reg8)		"# move\n"  1
reg8: LOADU8(reg8)		"# move\n"  1
reg8: LOADU4(indirmem58)	"ldm %aq,%0q\n"			1

reg2: CVFI2(dreg)		"d2i r52\nldr %cw,r52w\n"		2
reg4: CVFI4(dreg)		"d2i r52\nldr %cd,r52d\n"		2
reg8: CVFI8(dreg)		"d2i %cq\n"						1

stmt: reg1 ""
stmt: reg2 ""
stmt: reg4 ""
stmt: reg8 ""

mem56:	ADDRGP4					"%a"
mem56:	con						"%0"

regmem56: ADDI4(reg4, mem56)	"[%0][%1]"
regmem56: ADDP4(reg4, mem56)	"[%0][%1]"
regmem56: ADDU4(reg4, mem56)	"[%0][%1]"

addr56: reg4					"[%0]"
addr56: regmem56				"%0"
addr56: ADDRGP4					"[%a]"
addr56:	ADDI4(regmem56, mem56)	"%0[%1]"
addr56:	ADDP4(regmem56, mem56)	"%0[%1]"
addr56:	ADDU4(regmem56, mem56)	"%0[%1]"
addr56:	ADDI4(reg4, reg4)		"[%1][%0]"
addr56:	ADDP4(reg4, reg4)		"[%1][%0]"
addr56:	ADDU4(reg4, reg4)		"[%1][%0]"
addr56: ADDRFP4					"[r6][%a]"
addr56: ADDRLP4					"[r6][%a]"

lp4: ADDRLP4					"[r6][%a]"

indir561: INDIRI1(addr56)	"%0"
indir561: INDIRU1(addr56)	"%0"
indir562: INDIRI2(addr56)	"%0"
indir562: INDIRU2(addr56)	"%0"
indir564: INDIRI4(addr56)	"%0"
indir564: INDIRU4(addr56)	"%0"
indir568: INDIRI8(addr56)	"%0"
indir568: INDIRU8(addr56)	"%0"

mem5: reg4						"[%0]"
mem5: ADDI4(reg4, mem56)		"[%0][%1]"
mem5: ADDU4(reg4, mem56)		"[%0][%1]"
mem5: ADDRGP4					"[%a]"
mem5: ADDI4(reg4, ADDI4(mem56,mem56))	"[%0][%1]"
mem5: ADDU4(reg4, ADDU4(mem56,mem56))	"[%0][%1]"
mem5: ADDI4(reg4, reg4)			"[%0][%1]"
mem5: ADDU4(reg4, reg4)			"[%0][%1]"

indirmem51: INDIRI1(mem5)	"%0"
indirmem51: INDIRU1(mem5)	"%0"
indirmem52: INDIRI2(mem5)	"%0"
indirmem52: INDIRU2(mem5)	"%0"
indirmem54: INDIRI4(mem5)	"%0"
indirmem54: INDIRU4(mem5)	"%0"
indirmem54: INDIRP4(mem5)	"%0"
indirmem58: INDIRI8(mem5)	"%0"
indirmem58: INDIRU8(mem5)	"%0"

rc1:  reg1					"%0"
rc1:  con					"=%0"
rc2:  reg2					"%0"
rc2:  con					"=%0"
rc4:  reg4					"%0"
rc4:  con					"=%0"
rc8:  reg8					"%0"
rc8:  con					"=%0"

reg1: con					"ldi %cb,=%0\n"	1
reg2: con					"ldi %cw,=%0\n"	1
reg4: con					"ldi %c,=%0\n"	1
reg4: addr56				"lea %c,%0\n"	1
reg8: con					"ldi %cq,=%0\n"	1

reg1: LSHI1(reg1,rc1)		"?ldr %cb,%0\nshl %cb,%1\n"  2
reg1: LSHU1(reg1,rc1)		"?ldr %cb,%0\nshl %cb,%1\n"  2
reg1: LSHI1(reg1,rc2)		"?ldr %cb,%0\nshl %cb,%1\n"  2
reg1: LSHU1(reg1,rc2)		"?ldr %cb,%0\nshl %cb,%1\n"  2
reg1: LSHI1(reg1,rc4)		"?ldr %cb,%0\nshl %cb,%1\n"  2
reg1: LSHU1(reg1,rc4)		"?ldr %cb,%0\nshl %cb,%1\n"  2
reg1: LSHI1(reg1,rc8)		"?ldr %cb,%0\nshl %cb,%1\n"  2
reg1: LSHU1(reg1,rc8)		"?ldr %cb,%0\nshl %cb,%1\n"  2
reg2: LSHI2(reg2,rc1)		"?ldr %cw,%0\nshl %cw,%1\n"  2
reg2: LSHU2(reg2,rc1)		"?ldr %cw,%0\nshl %cw,%1\n"  2
reg2: LSHI2(reg2,rc2)		"?ldr %cw,%0\nshl %cw,%1\n"  2
reg2: LSHU2(reg2,rc2)		"?ldr %cw,%0\nshl %cw,%1\n"  2
reg2: LSHI2(reg2,rc4)		"?ldr %cw,%0\nshl %cw,%1\n"  2
reg2: LSHU2(reg2,rc4)		"?ldr %cw,%0\nshl %cw,%1\n"  2
reg2: LSHI2(reg2,rc8)		"?ldr %cw,%0\nshl %cw,%1\n"  2
reg2: LSHU2(reg2,rc8)		"?ldr %cw,%0\nshl %cw,%1\n"  2
reg4: LSHI4(reg4,rc1)		"?ldr %cd,%0\nshl %cd,%1\n"  2
reg4: LSHU4(reg4,rc1)		"?ldr %cd,%0\nshl %cd,%1\n"  2
reg4: LSHI4(reg4,rc2)		"?ldr %cd,%0\nshl %cd,%1\n"  2
reg4: LSHU4(reg4,rc2)		"?ldr %cd,%0\nshl %cd,%1\n"  2
reg4: LSHI4(reg4,rc4)		"?ldr %cd,%0\nshl %cd,%1\n"  2
reg4: LSHU4(reg4,rc4)		"?ldr %cd,%0\nshl %cd,%1\n"  2
reg4: LSHI4(reg4,rc8)		"?ldr %cd,%0\nshl %cd,%1\n"  2
reg4: LSHU4(reg4,rc8)		"?ldr %cd,%0\nshl %cd,%1\n"  2
reg8: LSHI8(reg8,rc1)		"?ldr %cq,%0\nshl %cq,%1\n"  2
reg8: LSHU8(reg8,rc1)		"?ldr %cq,%0\nshl %cq,%1\n"  2
reg8: LSHI8(reg8,rc2)		"?ldr %cq,%0\nshl %cq,%1\n"  2
reg8: LSHU8(reg8,rc2)		"?ldr %cq,%0\nshl %cq,%1\n"  2
reg8: LSHI8(reg8,rc4)		"?ldr %cq,%0\nshl %cq,%1\n"  2
reg8: LSHU8(reg8,rc4)		"?ldr %cq,%0\nshl %cq,%1\n"  2
reg8: LSHI8(reg8,rc8)		"?ldr %cq,%0\nshl %cq,%1\n"  2
reg8: LSHU8(reg8,rc8)		"?ldr %cq,%0\nshl %cq,%1\n"  2

reg1: RSHI1(reg1,rc1)		"?ldr %cb,%0\nshr %cb,%1\n"  2
reg1: RSHU1(reg1,rc1)		"?ldr %cb,%0\nusm\nshr %cb,%1\nsm\n"  4
reg1: RSHI1(reg1,rc2)		"?ldr %cb,%0\nshr %cb,%1\n"  2
reg1: RSHU1(reg1,rc2)		"?ldr %cb,%0\nusm\nshr %cb,%1\nsm\n"  4
reg1: RSHI1(reg1,rc4)		"?ldr %cb,%0\nshr %cb,%1\n"  2
reg1: RSHU1(reg1,rc4)		"?ldr %cb,%0\nusm\nshr %cb,%1\nsm\n"  4
reg1: RSHI1(reg1,rc8)		"?ldr %cb,%0\nshr %cb,%1\n"  2
reg1: RSHU1(reg1,rc8)		"?ldr %cb,%0\nusm\nshr %cb,%1\nsm\n"  4
reg2: RSHI2(reg2,rc1)		"?ldr %cw,%0\nshr %cw,%1\n"  2
reg2: RSHU2(reg2,rc1)		"?ldr %cw,%0\nusm\nshr %cw,%1\nsm\n"  4
reg2: RSHI2(reg2,rc2)		"?ldr %cw,%0\nshr %cw,%1\n"  2
reg2: RSHU2(reg2,rc2)		"?ldr %cw,%0\nusm\nshr %cw,%1\nsm\n"  4
reg2: RSHI2(reg2,rc4)		"?ldr %cw,%0\nshr %cw,%1\n"  2
reg2: RSHU2(reg2,rc4)		"?ldr %cw,%0\nusm\nshr %cw,%1\nsm\n"  4
reg2: RSHI2(reg2,rc8)		"?ldr %cw,%0\nshr %cw,%1\n"  2
reg2: RSHU2(reg2,rc8)		"?ldr %cw,%0\nusm\nshr %cw,%1\nsm\n"  4
reg4: RSHI4(reg4,rc1)		"?ldr %cd,%0\nshr %cd,%1\n"  2
reg4: RSHU4(reg4,rc1)		"?ldr %cd,%0\nusm\nshr %cd,%1\nsm\n"  4
reg4: RSHI4(reg4,rc2)		"?ldr %cd,%0\nshr %cd,%1\n"  2
reg4: RSHU4(reg4,rc2)		"?ldr %cd,%0\nusm\nshr %cd,%1\nsm\n"  4
reg4: RSHI4(reg4,rc4)		"?ldr %cd,%0\nshr %cd,%1\n"  2
reg4: RSHU4(reg4,rc4)		"?ldr %cd,%0\nusm\nshr %cd,%1\nsm\n"  4
reg4: RSHI4(reg4,rc8)		"?ldr %cd,%0\nshr %cd,%1\n"  2
reg4: RSHU4(reg4,rc8)		"?ldr %cd,%0\nusm\nshr %cd,%1\nsm\n"  4
reg8: RSHI8(reg8,rc1)		"?ldr %cq,%0\nshr %cq,%1\n"  2
reg8: RSHU8(reg8,rc1)		"?ldr %cq,%0\nusm\nshr %cq,%1\nsm\n"  4
reg8: RSHI8(reg8,rc2)		"?ldr %cq,%0\nshr %cq,%1\n"  2
reg8: RSHU8(reg8,rc2)		"?ldr %cq,%0\nusm\nshr %cq,%1\nsm\n"  4
reg8: RSHI8(reg8,rc4)		"?ldr %cq,%0\nshr %cq,%1\n"  2
reg8: RSHU8(reg8,rc4)		"?ldr %cq,%0\nusm\nshr %cq,%1\nsm\n"  4
reg8: RSHI8(reg8,rc8)		"?ldr %cq,%0\nshr %cq,%1\n"  2
reg8: RSHU8(reg8,rc8)		"?ldr %cq,%0\nusm\nshr %cq,%1\nsm\n"  4

reg1: ADDI1(reg1,con1)		"?ldr %cb,%0\ninc %cb\n"	2
reg1: ADDU1(reg1,con1)		"?ldr %cb,%0\ninc %cb\n"	2
reg2: ADDI2(reg2,con1)		"?ldr %cw,%0\ninc %cw\n"	2
reg2: ADDU2(reg2,con1)		"?ldr %cw,%0\ninc %cw\n"	2
reg4: ADDI4(reg4,con1)		"?ldr %cd,%0\ninc %cd\n"	2
reg4: ADDU4(reg4,con1)		"?ldr %cd,%0\ninc %cd\n"	2
reg4: ADDP4(reg4,con1)		"?ldr %cd,%0\ninc %cd\n"	2
reg8: ADDI8(reg8,con1)		"?ldr %cq,%0\ninc %cq\n"	2
reg8: ADDU8(reg8,con1)		"?ldr %cq,%0\ninc %cq\n"	2

reg1: ADDI1(reg1,conm1)		"?ldr %cb,%0\ndec %cb\n"	2
reg1: ADDU1(reg1,conm1)		"?ldr %cb,%0\ndec %cb\n"	2
reg2: ADDI2(reg2,conm1)		"?ldr %cw,%0\ndec %cw\n"	2
reg2: ADDU2(reg2,conm1)		"?ldr %cw,%0\ndec %cw\n"	2
reg4: ADDI4(reg4,conm1)		"?ldr %cd,%0\ndec %cd\n"	2
reg4: ADDU4(reg4,conm1)		"?ldr %cd,%0\ndec %cd\n"	2
reg4: ADDP4(reg4,conm1)		"?ldr %cd,%0\ndec %cd\n"	2
reg8: ADDI8(reg8,conm1)		"?ldr %cq,%0\ndec %cq\n"	2
reg8: ADDU8(reg8,conm1)		"?ldr %cq,%0\ndec %cq\n"	2

reg1: ADDI1(reg1,indir561)	"?ldr %cb,%0\nadd %cb,%1\n"		2
reg1: ADDI1(reg1,reg1)		"?ldr %cb,%0b\nadd %cb,%1b\n"	2
reg1: ADDI1(reg1,con)		"?ldr %cb,%0\nadd %cb,=%1\n"	2
reg1: ADDU1(reg1,indir561)	"?ldr %cb,%0\nadd %cb,%1\n"		2
reg1: ADDU1(reg1,reg1)		"?ldr %cb,%0b\nadd %cb,%1b\n"	2
reg1: ADDU1(reg1,con)		"?ldr %cb,%0\nadd %cb,=%1\n"	2

reg2: ADDI2(reg2,indir562)	"?ldr %cw,%0\nadd %cw,%1\n"		2
reg2: ADDI2(reg2,reg2)		"?ldr %cw,%0w\nadd %cw,%1w\n"	2
reg2: ADDI2(reg2,con)		"?ldr %cw,%0\nadd %cw,=%1\n"	2
reg2: ADDU2(reg2,indir562)	"?ldr %cw,%0\nadd %cw,%1\n"		2
reg2: ADDU2(reg2,reg2)		"?ldr %cw,%0w\nadd %cw,%1w\n"	2
reg2: ADDU2(reg2,con)		"?ldr %cw,%0\nadd %cw,=%1\n"	2

reg4: ADDI4(reg4,indir564)	"?ldr %cd,%0\nadd %cd,%1\n"		2
reg4: ADDI4(reg4,reg4)		"?ldr %cd,%0d\nadd %cd,%1\n"	2
reg4: ADDI4(reg4,con)		"?ldr %cd,%0\nadd %cd,=%1\n"	2
reg4: ADDU4(reg4,indir564)	"?ldr %cd,%0\nadd %cd,%1\n"		2
reg4: ADDU4(reg4,reg4)		"?ldr %cd,%0d\nadd %cd,%1\n"	2
reg4: ADDU4(reg4,con)		"?ldr %cd,%0\nadd %cd,=%1\n"	2

reg8: ADDI8(reg8,indir568)	"?ldr %cq,%0\nadd %cq,%1\n"		2
reg8: ADDI8(reg8,reg8)		"?ldr %cq,%0q\nadd %cq,%1q\n"	2
reg8: ADDI8(reg8,con)		"?ldr %cq,%0\nadd %cq,=%1\n"	2
reg8: ADDU8(reg8,indir568)	"?ldr %cq,%0\nadd %cq,%1\n"		2
reg8: ADDU8(reg8,reg8)		"?ldr %cq,%0q\nadd %cq,%1q\n"	2
reg8: ADDU8(reg8,con)		"?ldr %cq,%0\nadd %cq,=%1\n"	2

reg4: ADDP4(reg4, addr56)	"?ldr %cd,%0\nadd %cd,%1\n"		2
reg4: ADDP4(reg4,indir564)	"?ldr %cd,%0\nadd %cd,%1\n"		2
reg4: ADDP4(reg4,reg4)		"?ldr %cd,%0d\nadd %cd,%1\n"	2
reg4: ADDP4(reg4,con)		"?ldr %cd,%0\nadd %cd,=%1\n"	2

reg1: SUBI1(reg1,con1)		"?ldr %cb,%0\ndec %cb\n"	2
reg1: SUBU1(reg1,con1)		"?ldr %cb,%0\ndec %cb\n"	2
reg2: SUBI2(reg2,con1)		"?ldr %cw,%0\ndec %cw\n"	2
reg2: SUBU2(reg2,con1)		"?ldr %cw,%0\ndec %cw\n"	2
reg4: SUBI4(reg4,con1)		"?ldr %cd,%0\ndec %cd\n"	2
reg4: SUBU4(reg4,con1)		"?ldr %cd,%0\ndec %cd\n"	2
reg4: SUBP4(reg4,con1)		"?ldr %cd,%0\ndec %cd\n"	2
reg8: SUBI8(reg8,con1)		"?ldr %cq,%0\ndec %cq\n"	2
reg8: SUBU8(reg8,con1)		"?ldr %cq,%0\ndec %cq\n"	2

reg1: SUBI1(reg1,conm1)		"?ldr %cb,%0\ninc %cb\n"	2
reg1: SUBU1(reg1,conm1)		"?ldr %cb,%0\ninc %cb\n"	2
reg2: SUBI2(reg2,conm1)		"?ldr %cw,%0\ninc %cw\n"	2
reg2: SUBU2(reg2,conm1)		"?ldr %cw,%0\ninc %cw\n"	2
reg4: SUBI4(reg4,conm1)		"?ldr %cd,%0\ninc %cd\n"	2
reg4: SUBU4(reg4,conm1)		"?ldr %cd,%0\ninc %cd\n"	2
reg4: SUBP4(reg4,conm1)		"?ldr %cd,%0\ninc %cd\n"	2
reg8: SUBI8(reg8,conm1)		"?ldr %cq,%0\ninc %cq\n"	2
reg8: SUBU8(reg8,conm1)		"?ldr %cq,%0\ninc %cq\n"	2

reg1: SUBI1(reg1,indir561)	"?ldr %cb,%0\nsub %cb,%1\n"		2
reg1: SUBI1(reg1,reg1)		"?ldr %cb,%0b\nsub %cb,%1b\n"	2
reg1: SUBI1(reg1,con)		"?ldr %cb,%0\nsub %cb,=%1\n"	2
reg1: SUBU1(reg1,indir561)	"?ldr %cb,%0\nsub %cb,%1\n"		2
reg1: SUBU1(reg1,reg1)		"?ldr %cb,%0b\nsub %cb,%1b\n"	2
reg1: SUBU1(reg1,con)		"?ldr %cb,%0\nsub %cb,=%1\n"	2

reg2: SUBI2(reg2,indir562)	"?ldr %cw,%0\nsub %cw,%1\n"		2
reg2: SUBI2(reg2,reg2)		"?ldr %cw,%0w\nsub %cw,%1w\n"	2
reg2: SUBI2(reg2,con)		"?ldr %cw,%0\nsub %cw,=%1\n"	2
reg2: SUBU2(reg2,indir562)	"?ldr %cw,%0\nsub %cw,%1\n"		2
reg2: SUBU2(reg2,reg2)		"?ldr %cw,%0w\nsub %cw,%1w\n"	2
reg2: SUBU2(reg2,con)		"?ldr %cw,%0\nsub %cw,=%1\n"	2

reg4: SUBI4(reg4,indir564)	"?ldr %cd,%0\nsub %cd,%1\n"		2
reg4: SUBI4(reg4,reg4)		"?ldr %cd,%0d\nsub %cd,%1\n"	2
reg4: SUBI4(reg4,con)		"?ldr %cd,%0\nsub %cd,=%1\n"	2
reg4: SUBU4(reg4,indir564)	"?ldr %cd,%0\nsub %cd,%1\n"		2
reg4: SUBU4(reg4,reg4)		"?ldr %cd,%0d\nsub %cd,%1\n"	2
reg4: SUBU4(reg4,con)		"?ldr %cd,%0\nsub %cd,=%1\n"	2

reg8: SUBI8(reg8,indir568)	"?ldr %cq,%0\nsub %cq,%1\n"		2
reg8: SUBI8(reg8,reg8)		"?ldr %cq,%0q\nsub %cq,%1q\n"	2
reg8: SUBI8(reg8,con)		"?ldr %cq,%0\nsub %cq,=%1\n"	2
reg8: SUBU8(reg8,indir568)	"?ldr %cq,%0\nsub %cq,%1\n"		2
reg8: SUBU8(reg8,reg8)		"?ldr %cq,%0q\nsub %cq,%1q\n"	2
reg8: SUBU8(reg8,con)		"?ldr %cq,%0\nsub %cq,=%1\n"	2

reg4: SUBP4(reg4,indir564)	"?ldr %cd,%0\nsub %cd,%1\n"		2
reg4: SUBP4(reg4,reg4)		"?ldr %cd,%0d\nsub %cd,%1\n"	2
reg4: SUBP4(reg4,con)		"?ldr %cd,%0\nsub %cd,=%1\n"	2

reg2: MULI2(reg2,reg2)		"?ldr %cw,%0\nmul %w,%1w\n"		4
reg2: MULI2(reg2,con)		"?ldr %cw,%0\nmul %cw,=%1\n"	4
reg2: MULU2(reg2,reg2)		"?ldr %cw,%0\nmul %cw,%1w\n"	4
reg2: MULU2(reg2,con)		"?ldr %cw,%0\nmul %c,=%1\n"		4

reg4: MULI4(reg4,reg4)		"?ldr %cd,%0d\nmul %cd,%1d\n"	4
reg4: MULI4(reg4,con)		"?ldr %cd,%0\nmul %cd,=%1\n"	4
reg4: MULU4(reg4,reg4)		"?ldr %cd,%0d\nmul %cd,%1d\n"	4
reg4: MULU4(reg4,con)		"?ldr %cd,%0\nmul %cd,=%1\n"	4

reg8: MULI8(reg8,reg8)		"?ldr %cq,%0q\nmul %cq,%1q\n"	4
reg8: MULI8(reg8,con)		"?ldr %cq,%0q\nmul %cq,=%1\n"	4
reg8: MULU8(reg8,reg8)		"?ldr %cq,%0q\nmul %cq,%1q\n"	4
reg8: MULU8(reg8,con)		"?ldr %cq,%0\nmul %cq,=%1\n"	4

reg4: MULI4(reg4,con2)		"?ldr %cd,%0d\nshl %cd,=1\n"	2
reg4: MULI4(reg4,con4)		"?ldr %cd,%0d\nshl %cd,=2\n"	2
reg4: MULI4(reg4,con8)		"?ldr %cd,%0d\nshl %cd,=3\n"	2
reg4: MULI4(reg4,con16)		"?ldr %cd,%0d\nshl %cd,=4\n"	2
reg4: MULI4(reg4,con32)		"?ldr %cd,%0d\nshl %cd,=5\n"	2
reg4: MULI4(reg4,con64)		"?ldr %cd,%0d\nshl %cd,=6\n"	2
reg4: MULI4(reg4,con128)	"?ldr %cd,%0d\nshl %cd,=7\n"	2
reg4: MULI4(reg4,con256)	"?ldr %cd,%0d\nshl %cd,=8\n"	2
reg4: MULU4(reg4,con2)		"?ldr %cd,%0d\nshl %cd,=1\n"	2
reg4: MULU4(reg4,con4)		"?ldr %cd,%0d\nshl %cd,=2\n"	2
reg4: MULU4(reg4,con8)		"?ldr %cd,%0d\nshl %cd,=3\n"	2
reg4: MULU4(reg4,con16)		"?ldr %cd,%0d\nshl %cd,=4\n"	2
reg4: MULU4(reg4,con32)		"?ldr %cd,%0d\nshl %cd,=5\n"	2
reg4: MULU4(reg4,con64)		"?ldr %cd,%0d\nshl %cd,=6\n"	2
reg4: MULU4(reg4,con128)	"?ldr %cd,%0d\nshl %cd,=7\n"	2
reg4: MULU4(reg4,con256)	"?ldr %cd,%0d\nshl %cd,=8\n"	2

reg2: DIVI2(reg2,reg2)		"?ldr %cw,%0w\ndiv %cw,%1w\n"			4
reg2: DIVU2(reg2,reg2)		"?ldr %cw,%0w\nusm\ndiv %cw,%1w\nsm\n"	6
reg2: DIVI2(reg2,con)		"?ldr %cw,%0\ndiv %cw,=%1\n"			4
reg2: DIVU2(reg2,con)		"?ldr %cw,%0\nusm\ndiv %cw,=%1\nsm\n"	6
reg4: DIVI4(reg4,reg4)		"?ldr %cd,%0d\ndiv %cd,%1d\n"			4
reg4: DIVU4(reg4,reg4)		"?ldr %cd,%0d\nusm\ndiv %cd,%1d\nsm\n"	6
reg4: DIVI4(reg4,con)		"?ldr %cd,%0\ndiv %c,=%1\n"				4
reg4: DIVU4(reg4,con)		"?ldr %cd,%0\nusm\ndiv %c,=%1\nsm\n"	6
reg8: DIVI8(reg8,reg8)		"?ldr %cq,%0q\ndiv %cq,%1q\n"			2
reg8: DIVI8(reg8,con)		"?ldr %cq,%0\ndiv %cq,=%1\n"			2
reg8: DIVU8(reg8,reg8)		"?ldr %cq,%0q\nusm\ndiv %cq,%1q\nsm\n"	4
reg8: DIVU8(reg8,con)		"?ldr %cq,%0\nusm\ndiv %cq,=%1\nsm\n"	4

reg4: DIVI4(reg4,con2)		"?ldr %cd,%0\nshr %cd,=1\n"				2
reg4: DIVI4(reg4,con4)		"?ldr %cd,%0\nshr %cd,=2\n"				2
reg4: DIVI4(reg4,con8)		"?ldr %cd,%0\nshr %cd,=3\n"				2
reg4: DIVI4(reg4,con16)		"?ldr %cd,%0\nshr %cd,=4\n"				2
reg4: DIVI4(reg4,con32)		"?ldr %cd,%0\nshr %cd,=5\n"				2
reg4: DIVI4(reg4,con64)		"?ldr %cd,%0\nshr %cd,=6\n"				2
reg4: DIVI4(reg4,con128)	"?ldr %cd,%0\nshr %cd,=7\n"				2
reg4: DIVI4(reg4,con256)	"?ldr %cd,%0\nshr %cd,=8\n"				2
reg4: DIVU4(reg4,con2)		"?ldr %cd,%0\nusm\nshr %cd,=1\nsm\n"	3
reg4: DIVU4(reg4,con4)		"?ldr %cd,%0\nusm\nshr %cd,=2\nsm\n"	3
reg4: DIVU4(reg4,con8)		"?ldr %cd,%0\nusm\nshr %cd,=3\nsm\n"	3
reg4: DIVU4(reg4,con16)		"?ldr %cd,%0\nusm\nshr %cd,=4\nsm\n"	3
reg4: DIVU4(reg4,con32)		"?ldr %cd,%0\nusm\nshr %cd,=5\nsm\n"	3
reg4: DIVU4(reg4,con64)		"?ldr %cd,%0\nusm\nshr %cd,=6\nsm\n"	3
reg4: DIVU4(reg4,con128)	"?ldr %cd,%0\nusm\nshr %cd,=7\nsm\n"	3
reg4: DIVU4(reg4,con256)	"?ldr %cd,%0\nusm\nshr %cd,=8\nsm\n"	3

reg2: MODI2(reg2,reg2)		"?ldr %cw,%0\nmod %cw,%1w\n"			2
reg2: MODI2(reg2,con)		"?ldr %cw,%0\nmod %cw,=%1\n"			2
reg2: MODU2(reg2,reg2)		"?ldr %cw,%0\nusm\nmod %cw,%1\nsm\n"	4
reg2: MODU2(reg2,con)		"?ldr %cw,%0\nusm\nmod %cw,=%1\nsm\n"	4

reg4: MODI4(reg4,reg4)		"?ldr %cd,%0\nmod %cd,%1\n"				2
reg4: MODI4(reg4,con)		"?ldr %cd,%0\nmod %cd,=%1\n"			2
reg4: MODU4(reg4,reg4)		"?ldr %cd,%0\nusm\nmod %cd,%1\nsm\n"	4
reg4: MODU4(reg4,con)		"?ldr %cd,%0\nusm\nmod %cd,=%1\nsm\n"	4

reg8: MODI8(reg8,reg8)		"?ldr %cq,%0q\nmod %cq,%1q\n"			2
reg8: MODI8(reg8,con)		"?ldr %cq,%0\nmod %cq,=%1\n"			2
reg8: MODU8(reg8,reg8)		"?ldr %cq,%0q\nusm\nmod %cq,%1q\nsm\n"	4
reg8: MODU8(reg8,con)		"?ldr %cq,%0\nusm\nmod %cq,=%1\nsm\n"	4

reg4: MODI4(reg4,con2)		"?ldr %cd,%0\nand %cd,=0x001\n"			2
reg4: MODI4(reg4,con4)		"?ldr %cd,%0\nand %cd,=0x003\n"			2
reg4: MODI4(reg4,con8)		"?ldr %cd,%0\nand %cd,=0x007\n"			2
reg4: MODI4(reg4,con16)		"?ldr %cd,%0\nand %cd,=0x00F\n"			2
reg4: MODI4(reg4,con32)		"?ldr %cd,%0\nand %cd,=0x01F\n"			2
reg4: MODI4(reg4,con64)		"?ldr %cd,%0\nand %cd,=0x03F\n"			2
reg4: MODI4(reg4,con128)	"?ldr %cd,%0\nand %cd,=0x07F\n"			2
reg4: MODI4(reg4,con256)	"?ldr %cd,%0\nand %cd,=0x0FF\n"			2
reg4: MODU4(reg4,con2)		"?ldr %cd,%0\nand %cd,=0x001\n"			2
reg4: MODU4(reg4,con4)		"?ldr %cd,%0\nand %cd,=0x003\n"			2
reg4: MODU4(reg4,con8)		"?ldr %cd,%0\nand %cd,=0x007\n"			2
reg4: MODU4(reg4,con16)		"?ldr %cd,%0\nand %cd,=0x00F\n"			2
reg4: MODU4(reg4,con32)		"?ldr %cd,%0\nand %cd,=0x01F\n"			2
reg4: MODU4(reg4,con64)		"?ldr %cd,%0\nand %cd,=0x03F\n"			2
reg4: MODU4(reg4,con128)	"?ldr %cd,%0\nand %cd,=0x07F\n"			2
reg4: MODU4(reg4,con256)	"?ldr %cd,%0\nand %cd,=0x0FF\n"			2

reg2: BANDI2(reg2,indir562)	"?ldr %cw,%0\nand %cw,%1\n"				2
reg2: BANDI2(reg2,reg2)		"?ldr %cw,%0w\nand %cw,%1w\n"			2
reg2: BANDI2(reg2,con)		"?ldr %cw,%0\nand %cw,=%1\n"			2
reg2: BANDU2(reg2,indir562)	"?ldr %cw,%0\nand %cw,%1\n"				2
reg2: BANDU2(reg2,reg2)		"?ldr %cw,%0w\nand %cw,%1w\n"			2
reg2: BANDU2(reg2,con)		"?ldr %cw,%0\nand %cw,=%1\n"			2

reg4: BANDI4(reg4,indir564)	"?ldr %cd,%0\nand %cd,%1\n"				2
reg4: BANDI4(reg4,reg4)		"?ldr %cd,%0d\nand %cd,%1d\n"			2
reg4: BANDI4(reg4,con)		"?ldr %cd,%0\nand %cd,=%1\n"			2
reg4: BANDU4(reg4,indir564)	"?ldr %cd,%0\nand %cd,%1\n"				2
reg4: BANDU4(reg4,reg4)		"?ldr %cd,%0d\nand %cd,%1d\n"			2
reg4: BANDU4(reg4,con)		"?ldr %cd,%0\nand %cd,=%1\n"			2

reg8: BANDI8(reg8,indir568)	"?ldr %cq,%0\nand %cq,%1\n"				2
reg8: BANDI8(reg8,reg8)		"?ldr %cq,%0q\nand %cq,%1q\n"			2
reg8: BANDI8(reg8,con)		"?ldr %cq,%0\nand %cq,=%1\n"			2
reg8: BANDU8(reg8,indir568)	"?ldr %cq,%0\nand %cq,%1\n"				2
reg8: BANDU8(reg8,reg8)		"?ldr %cq,%0q\nand %cq,%1q\n"			2
reg8: BANDU8(reg8,con)		"?ldr %cq,%0\nand %cq,=0x%1\n"			2

reg2: BORI2(reg2,indir562)	"?ldr %cw,%0\nor %cw,%1\n"				2
reg2: BORI2(reg2,reg2)		"?ldr %cw,%0w\nor %cw,%1w\n"			2
reg2: BORI2(reg2,con)		"?ldr %cw,%0\nor %cw,=%1\n"				2
reg2: BORU2(reg2,indir562)	"?ldr %cw,%0\nor %cw,%1\n"				2
reg2: BORU2(reg2,reg2)		"?ldr %cw,%0w\nor %cw,%1w\n"			2
reg2: BORU2(reg2,con)		"?ldr %cw,%0\nor %cw,=%1\n"				2

reg4: BORI4(reg4,indir564)	"?ldr %cd,%0\nor %cd,%1\n"				2
reg4: BORI4(reg4,reg4)		"?ldr %cd,%0d\nor %cd,%1d\n"			2
reg4: BORI4(reg4,con)		"?ldr %cd,%0\nor %cd,=%1\n"				2
reg4: BORU4(reg4,indir564)	"?ldr %cd,%0\nor %cd,%1\n"				2
reg4: BORU4(reg4,reg4)		"?ldr %cd,%0d\nor %cd,%1d\n"			2
reg4: BORU4(reg4,con)		"?ldr %cd,%0\nor %cd,=%1\n"				2

reg8: BORI8(reg8,indir568)	"?ldr %cq,%0\nor %cq,%1\n"				2
reg8: BORI8(reg8,reg8)		"?ldr %cq,%0q\nor %cq,%1q\n"			2
reg8: BORI8(reg8,con)		"?ldr %cq,%0\nor %cq,=%1\n"				2
reg8: BORU8(reg8,indir568)	"?ldr %cq,%0\nor %cq,%1\n"				2
reg8: BORU8(reg8,reg8)		"?ldr %cq,%0q\nor %cq,%1q\n"			2
reg8: BORU8(reg8,con)		"?ldr %cq,%0\nor %cq,=%1\n"				2

reg2: BXORI2(reg2,indir562)	"?ldr %cw,%0\nxor %cw,%1\n"				2
reg2: BXORI2(reg2,reg2)		"?ldr %cw,%0w\nxor %cw,%1w\n"			2
reg2: BXORI2(reg2,con)		"?ldr %cw,%0\nxor %cw,=%1\n"			2
reg2: BXORU2(reg2,indir562)	"?ldr %cw,%0\nxor %cw,%1\n"				2
reg2: BXORU2(reg2,reg2)		"?ldr %cw,%0w\nxor %cw,%1w\n"			2
reg2: BXORU2(reg2,con)		"?ldr %cw,%0\nxor %cw,=%1\n"			2

reg4: BXORI4(reg4,indir564)	"?ldr %cd,%0\nxor %cd,%1\n"				2
reg4: BXORI4(reg4,reg4)		"?ldr %cd,%0d\nxor %cd,%1d\n"			2
reg4: BXORI4(reg4,con)		"?ldr %cd,%0\nxor %cd,=%1\n"			2
reg4: BXORU4(reg4,indir564)	"?ldr %cd,%0\nxor %cd,%1\n"				2
reg4: BXORU4(reg4,reg4)		"?ldr %cd,%0d\nxor %cd,%1d\n"			2
reg4: BXORU4(reg4,con)		"?ldr %cd,%0\nxor %cd,=%1\n"			2

reg8: BXORI8(reg8,indir568)	"?ldr %cq,%0\nxor %cq,%1\n"				2
reg8: BXORI8(reg8,reg8)		"?ldr %cq,%0q\nxor %cq,%1q\n"			2
reg8: BXORI8(reg8,con)		"?ldr %cq,%0\nxor %cq,=%1\n"			2
reg8: BXORU8(reg8,indir568)	"?ldr %cq,%0\nxor %cq,%1\n"				2
reg8: BXORU8(reg8,reg8)		"?ldr %cq,%0q\nxor %cq,%1q\n"			2
reg8: BXORU8(reg8,con)		"?ldr %cq,%0\nxor %cq,=%1\n"			2

reg2: BCOMI2(reg2)			"?ldr %cw,%0\nnot %cw\n"  2
reg2: BCOMU2(reg2)			"?ldr %cw,%0\nnot %cw\n"  2
reg4: BCOMI4(reg4)			"?ldr %cd,%0\nnot %cd\n"  2
reg4: BCOMU4(reg4)			"?ldr %cd,%0\nnot %cd\n"  2
reg8: BCOMI8(reg8)			"?ldr %cq,%0\nnot %cq\n"  2
reg8: BCOMU8(reg8)			"?ldr %cq,%0\nnot %cq\n"  2

reg2: NEGI2(reg2)			"?ldr %cw,%0\nneg %cw\n"  2
reg4: NEGI4(reg4)			"?ldr %cd,%0\nneg %cd\n"  2
reg8: NEGI8(reg8)			"?ldr %cq,%0\nneg %cq\n"  2

stmt: EQI1(reg1,con0)		"tst %0b\nlje =%a\n"	2
stmt: EQU1(reg1,con0)		"tst %0b\nlje =%a\n"	2
stmt: EQI2(reg2,con0)		"tst %0w\nlje =%a\n"	2
stmt: EQU2(reg2,con0)		"tst %0w\nlje =%a\n"	2
stmt: EQI4(reg4,con0)		"tst %0d\nlje =%a\n"	2
stmt: EQU4(reg4,con0)		"tst %0d\nlje =%a\n"	2
stmt: EQI8(reg8,con0)		"tst %0q\nlje =%a\n"	2
stmt: EQU8(reg8,con0)		"tst %0q\nlje =%a\n"	2

stmt: EQI1(indir561,con0)	"tst %0b\nlje =%a\n"	2
stmt: EQU1(indir561,con0)	"tst %0b\nlje =%a\n"	2
stmt: EQI2(indir562,con0)	"tst %0w\nlje =%a\n"	2
stmt: EQU2(indir562,con0)	"tst %0w\nlje =%a\n"	2
stmt: EQI4(indir564,con0)	"tst %0d\nlje =%a\n"	2
stmt: EQU4(indir564,con0)	"tst %0d\nlje =%a\n"	2
stmt: EQI8(indir568,con0)	"tst %0q\nlje =%a\n"	2
stmt: EQU8(indir568,con0)	"tst %0q\nlje =%a\n"	2

stmt: EQI1(reg1,indir561)	"cmp %0b,%1\nlje =%a\n"	2
stmt: EQU1(reg1,indir561)	"cmp %0b,%1\nlje =%a\n"	2
stmt: EQI2(reg2,indir562)	"cmp %0w,%1\nlje =%a\n"	2
stmt: EQU2(reg2,indir562)	"cmp %0w,%1\nlje =%a\n"	2
stmt: EQI4(reg4,indir564)	"cmp %0d,%1\nlje =%a\n"	2
stmt: EQU4(reg4,indir564)	"cmp %0d,%1\nlje =%a\n"	2
stmt: EQI8(reg8,indir568)	"cmp %0q,%1\nlje =%a\n"	2
stmt: EQU8(reg8,indir568)	"cmp %0q,%1\nlje =%a\n"	2

stmt: EQI1(reg1,rc1)		"cmp %0b,%1\nlje =%a\n"	2
stmt: EQU1(reg1,rc1)		"cmp %0b,%1\nlje =%a\n"	2
stmt: EQI2(reg2,rc2)		"cmp %0w,%1\nlje =%a\n"	2
stmt: EQU2(reg2,rc2)		"cmp %0w,%1\nlje =%a\n"	2
stmt: EQI4(reg4,rc4)		"cmp %0d,%1\nlje =%a\n"	2
stmt: EQU4(reg4,rc4)		"cmp %0d,%1\nlje =%a\n"	2
stmt: EQI8(reg8,rc8)		"cmp %0q,%1\nlje =%a\n"	2
stmt: EQU8(reg8,rc8)		"cmp %0q,%1\nlje =%a\n"	2

stmt: NEI1(reg1,con0)		"tst %0b\nljne =%a\n"	2
stmt: NEU1(reg1,con0)		"tst %0b\nljne =%a\n"	2
stmt: NEI2(reg2,con0)		"tst %0w\nljne =%a\n"	2
stmt: NEU2(reg2,con0)		"tst %0w\nljne =%a\n"	2
stmt: NEI4(reg4,con0)		"tst %0d\nljne =%a\n"	2
stmt: NEU4(reg4,con0)		"tst %0d\nljne =%a\n"	2
stmt: NEI8(reg8,con0)		"tst %0q\nljne =%a\n"	2
stmt: NEU8(reg8,con0)		"tst %0q\nljne =%a\n"	2

stmt: NEI1(indir561,con0)	"tst %0b\nljne =%a\n"	2
stmt: NEU1(indir561,con0)	"tst %0b\nljne =%a\n"	2
stmt: NEI2(indir562,con0)	"tst %0w\nljne =%a\n"	2
stmt: NEU2(indir562,con0)	"tst %0w\nljne =%a\n"	2
stmt: NEI4(indir564,con0)	"tst %0d\nljne =%a\n"	2
stmt: NEU4(indir564,con0)	"tst %0d\nljne =%a\n"	2
stmt: NEI8(indir568,con0)	"tst %0q\nljne =%a\n"	2
stmt: NEU8(indir568,con0)	"tst %0q\nljne =%a\n"	2

stmt: NEI1(reg1,indir561)	"cmp %0b,%1\nljne =%a\n"	2
stmt: NEU1(reg1,indir561)	"cmp %0b,%1\nljne =%a\n"	2
stmt: NEI2(reg2,indir562)	"cmp %0w,%1\nljne =%a\n"	2
stmt: NEU2(reg2,indir562)	"cmp %0w,%1\nljne =%a\n"	2
stmt: NEI4(reg4,indir564)	"cmp %0d,%1\nljne =%a\n"	2
stmt: NEU4(reg4,indir564)	"cmp %0d,%1\nljne =%a\n"	2
stmt: NEI8(reg8,indir568)	"cmp %0q,%1\nljne =%a\n"	2
stmt: NEU8(reg8,indir568)	"cmp %0q,%1\nljne =%a\n"	2

stmt: NEI1(reg1,rc1)		"cmp %0b,%1\nljne =%a\n"	2
stmt: NEU1(reg1,rc1)		"cmp %0b,%1\nljne =%a\n"	2
stmt: NEI2(reg2,rc2)		"cmp %0w,%1\nljne =%a\n"	2
stmt: NEU2(reg2,rc2)		"cmp %0w,%1\nljne =%a\n"	2
stmt: NEI4(reg4,rc4)		"cmp %0d,%1\nljne =%a\n"	2
stmt: NEU4(reg4,rc4)		"cmp %0d,%1\nljne =%a\n"	2
stmt: NEI8(reg8,rc8)		"cmp %0q,%1\nljne =%a\n"	2
stmt: NEU8(reg8,rc8)		"cmp %0q,%1\nljne =%a\n"	2

stmt: GEU1(reg1,indir561)		"usm\ncmp %0b,%1\nsm\nljge =%a\n"	4
stmt: GEU1(reg1,rc1)			"usm\ncmp %0b,%1\nsm\nljge =%a\n"	4
stmt: GEI1(reg1,indir561)		"cmp %0b,%1\nljge =%a\n"			2
stmt: GEI1(reg1,rc1)			"cmp %0b,%1\nljge =%a\n"			2
stmt: GEU2(reg2,indir562)		"usm\ncmp %0w,%1\nsm\nljge =%a\n"	4
stmt: GEU2(reg2,rc2)			"usm\ncmp %0w,%1\nsm\nljge =%a\n"	4
stmt: GEI2(reg2,indir562)		"cmp %0w,%1\nljge =%a\n"			2
stmt: GEI2(reg2,rc2)			"cmp %0w,%1\nljge =%a\n"			2
stmt: GEU4(reg4,indir564)		"usm\ncmp %0d,%1\nsm\nljge =%a\n"	4
stmt: GEU4(reg4,rc4)			"usm\ncmp %0d,%1\nsm\nljge =%a\n"	4
stmt: GEI4(reg4,indir564)		"cmp %0d,%1\nljge =%a\n"			2
stmt: GEI4(reg4,rc4)			"cmp %0d,%1\nljge =%a\n"			2
stmt: GEU8(reg8,indir568)		"usm\ncmp %0q,%1\nsm\nljge =%a\n"	4
stmt: GEU8(reg8,rc8)			"usm\ncmp %0q,%1\nsm\nljge =%a\n"	4
stmt: GEI8(reg8,indir568)		"cmp %0q,%1\nljge =%a\n"			2
stmt: GEI8(reg8,rc8)			"cmp %0q,%1\nljge =%a\n"			2

stmt: GTU1(reg1,indir561)		"usm\ncmp %0b,%1\nsm\nljg =%a\n"	4
stmt: GTU1(reg1,rc1)			"usm\ncmp %0b,%1\nsm\nljg =%a\n"	4
stmt: GTI1(reg1,indir561)		"cmp %0b,%1\nljg =%a\n"				2
stmt: GTI1(reg1,rc1)			"cmp %0b,%1\nljg =%a\n"				2
stmt: GTU2(reg2,indir562)		"usm\ncmp %0w,%1\nsm\nljg =%a\n"	4
stmt: GTU2(reg2,rc2)			"usm\ncmp %0w,%1\nsm\nljg =%a\n"	4
stmt: GTI2(reg2,indir562)		"cmp %0w,%1\nljg =%a\n"				2
stmt: GTI2(reg2,rc2)			"cmp %0w,%1\nljg =%a\n"				2
stmt: GTU4(reg4,indir564)		"usm\ncmp %0d,%1\nsm\nljg =%a\n"	4
stmt: GTU4(reg4,rc4)			"usm\ncmp %0d,%1\nsm\nljg =%a\n"	4
stmt: GTI4(reg4,indir564)		"cmp %0d,%1\nljg =%a\n"				2
stmt: GTI4(reg4,rc4)			"cmp %0d,%1\nljg =%a\n"				2
stmt: GTU8(reg8,indir568)		"usm\ncmp %0q,%1\nsm\nljg =%a\n"	4
stmt: GTU8(reg8,rc8)			"usm\ncmp %0q,%1\nsm\nljg =%a\n"	4
stmt: GTI8(reg8,indir568)		"cmp %0q,%1\nljg =%a\n"				2
stmt: GTI8(reg8,rc8)			"cmp %0q,%1\nljg =%a\n"				2

stmt: LEU1(reg1,indir561)		"usm\ncmp %0b,%1\nsm\nljle =%a\n"	4
stmt: LEU1(reg1,rc1)			"usm\ncmp %0b,%1\nsm\nljle =%a\n"	4
stmt: LEI1(reg1,indir561)		"cmp %0b,%1\nljle =%a\n"			2
stmt: LEI1(reg1,rc1)			"cmp %0b,%1\nljle =%a\n"			2
stmt: LEU2(reg2,indir562)		"usm\ncmp %0w,%1\nsm\nljle =%a\n"	4
stmt: LEU2(reg2,rc2)			"usm\ncmp %0w,%1\nsm\nljle =%a\n"	4
stmt: LEI2(reg2,indir562)		"cmp %0w,%1\nljle =%a\n"			2
stmt: LEI2(reg2,rc2)			"cmp %0w,%1\nljle =%a\n"			2
stmt: LEU4(reg4,indir564)		"usm\ncmp %0d,%1\nsm\nljle =%a\n"	4
stmt: LEU4(reg4,rc4)			"usm\ncmp %0d,%1\nsm\nljle =%a\n"	4
stmt: LEI4(reg4,indir564)		"cmp %0d,%1\nljle =%a\n"			2
stmt: LEI4(reg4,rc4)			"cmp %0d,%1\nljle =%a\n"			2
stmt: LEU8(reg8,indir568)		"usm\ncmp %0q,%1\nsm\nljle =%a\n"	4
stmt: LEU8(reg8,rc8)			"usm\ncmp %0q,%1\nsm\nljle =%a\n"	4
stmt: LEI8(reg8,indir568)		"cmp %0q,%1\nljle =%a\n"			2
stmt: LEI8(reg8,rc8)			"cmp %0q,%1\nljle =%a\n"			2

stmt: LTU1(reg1,indir561)		"usm\ncmp %0b,%1\nsm\nljl =%a\n"	4
stmt: LTU1(reg1,rc1)			"usm\ncmp %0b,%1\nsm\nljl =%a\n"	4
stmt: LTI1(reg1,indir561)		"cmp %0b,%1\nljl =%a\n"				2
stmt: LTI1(reg1,rc1)			"cmp %0b,%1\nljl =%a\n"				2
stmt: LTU2(reg2,indir562)		"usm\ncmp %0w,%1\nsm\nljl =%a\n"	4
stmt: LTU2(reg2,rc2)			"usm\ncmp %0w,%1\nsm\nljl =%a\n"	4
stmt: LTI2(reg2,indir562)		"cmp %0w,%1\nljl =%a\n"				2
stmt: LTI2(reg2,rc2)			"cmp %0w,%1\nljl =%a\n"				2
stmt: LTU4(reg4,indir564)		"usm\ncmp %0d,%1\nsm\nljl =%a\n"	4
stmt: LTU4(reg4,rc4)			"usm\ncmp %0d,%1\nsm\nljl =%a\n"	4
stmt: LTI4(reg4,indir564)		"cmp %0d,%1\nljl =%a\n"				2
stmt: LTI4(reg4,rc4)			"cmp %0d,%1\nljl =%a\n"				2
stmt: LTU8(reg8,indir568)		"usm\ncmp %0q,%1\nsm\nljl =%a\n"	4
stmt: LTU8(reg8,rc8)			"usm\ncmp %0q,%1\nsm\nljl =%a\n"	4
stmt: LTI8(reg8,indir568)		"cmp %0q,%1\nljl =%a\n"				2
stmt: LTI8(reg8,rc8)			"cmp %0q,%1\nljl =%a\n"				2

reg1: CVII1(reg1)				"?ldr %cb,%0\n"					1
reg1: CVII1(INDIRI1(addr56))	"ldm %cb,%0\n"					1
reg1: CVII1(reg2)				"?ldr %cb,%0\n"					1
reg1: CVII1(INDIRI2(addr56))	"ldm %cb,%0\n"					1
reg1: CVII1(reg4)				"?ldr %cb,%0\n"					1
reg1: CVII1(INDIRI4(addr56))	"ldm %cb,%0\n"					1
reg1: CVII1(reg8)				"?ldr %cb,%0\n"					1
reg1: CVII1(INDIRI8(addr56))	"ldm %cb,%0\n"					1

reg2: CVII2(reg1)				"?ldr %cb,%0\ncbw %c\n"			2
reg2: CVII2(INDIRI1(addr56))	"ldm %cb,%0\ncbw %c\n"			2
reg2: CVII2(reg2)				"?ldr %cw,%0\n"					1
reg2: CVII2(INDIRI2(addr56))	"ldm %cw,%0\n"					1
reg2: CVII2(reg4)				"?ldr %cw,%0\n"					1
reg2: CVII2(INDIRI4(addr56))	"ldm %cw,%0\n"					1
reg2: CVII2(reg8)				"?ldr %cw,%0\n"					1
reg2: CVII2(INDIRI8(addr56))	"ldm %cw,%0\n"					1

reg4: CVII4(reg1)				"?ldr %cb,%0\ncbd %c\n"			2
reg4: CVII4(INDIRI1(addr56))	"ldm %cb,%0\ncbd %c\n"			2
reg4: CVII4(reg2)				"?ldr %cw,%0\ncwd %c\n"			2
reg4: CVII4(INDIRI2(addr56))	"ldm %cw,%0\ncwd %c\n"			2
reg4: CVII4(reg4)				"?ldr %cd,%0\n"					1
reg4: CVII4(INDIRI4(addr56))	"ldm %cd,%0\n"					1
reg4: CVII4(reg8)				"?ldr %cd,%0\n"					1
reg4: CVII4(INDIRI8(addr56))	"ldm %cd,%0\n"					1

reg8: CVII8(reg1)				"?ldr %cb,%0\ncbq %c\n"			2
reg8: CVII8(INDIRI1(addr56))	"ldm %cb,%0\ncbq %c\n"			2
reg8: CVII8(reg2)				"?ldr %cw,%0\ncwq %c\n"			2
reg8: CVII8(INDIRI2(addr56))	"ldm %cw,%0\ncwq %c\n"			2
reg8: CVII8(reg4)				"?ldr %cd,%0\ncdq %c\n"			1
reg8: CVII8(INDIRI4(addr56))	"ldm %cd,%0\ncdq %c\n"			1
reg8: CVII8(reg8)				"?ldr %cq,%0\n"					1
reg8: CVII8(INDIRI8(addr56))	"ldm %cq,%0\n"					1

reg1: CVUU1(reg1)				"?ldr %cb,%0\n"					1
reg1: CVUU1(INDIRI1(addr56))	"ldm %cb,%0\n"					1
reg1: CVUU1(reg2)				"?ldr %cb,%0\n"					1
reg1: CVUU1(INDIRI2(addr56))	"ldm %cb,%0\n"					1
reg1: CVUU1(reg4)				"?ldr %cb,%0\n"					1
reg1: CVUU1(INDIRI4(addr56))	"ldm %cb,%0\n"					1
reg1: CVUU1(reg8)				"?ldr %cb,%0\n"					1
reg1: CVUU1(INDIRI8(addr56))	"ldm %cb,%0\n"					1

reg2: CVUU2(reg1)				"ldr r110,%0\nlde %cw,r110\n"	2
reg2: CVUU2(INDIRI1(addr56))	"ldm r110,%0\nlde %cw,r110\n"	2
reg2: CVUU2(reg2)				"?ldr %cw,%0\n"					1
reg2: CVUU2(INDIRI2(addr56))	"ldm %cw,%0\n"					1
reg2: CVUU2(reg4)				"?ldr %cw,%0\n"					1
reg2: CVUU2(INDIRI4(addr56))	"ldm %cw,%0\n"					1
reg2: CVUU2(reg8)				"?ldr %cw,%0\n"					1
reg2: CVUU2(INDIRI8(addr56))	"ldm %cw,%0\n"					1

reg4: CVUU4(reg1)				"ldr r110,%0\nlde %cd,r110\n"	2
reg4: CVUU4(INDIRI1(addr56))	"ldm r110,%0\nlde %cd,r110\n"	2
reg4: CVUU4(reg2)				"ldr r51,%0\nlde %cd,r51\n"		2
reg4: CVUU4(INDIRI2(addr56))	"ldm r51,%0\nlde %cd,r51\n"		2
reg4: CVUU4(reg4)				"?ldr %cd,%0\n"					1
reg4: CVUU4(INDIRI4(addr56))	"ldm %cd,%0\n"					1
reg4: CVUU4(reg8)				"?ldr %cd,%0\n"					1
reg4: CVUU4(INDIRI8(addr56))	"ldm %cd,%0\n"					1

reg4: CVPU4(reg4)				"?ldr %cd,%0\n"					1
reg4: CVUP4(reg4)				"?ldr %cd,%0\n"					1

reg8: CVUU8(reg1)				"ldr r110,%0\nlde %cq,r110\n"	2
reg8: CVUU8(INDIRI1(addr56))	"ldm r110,%0\nlde %cq,r110\n"	2
reg8: CVUU8(reg2)				"ldr r51,%0\nlde %cq,r51\n"		2
reg8: CVUU8(INDIRI2(addr56))	"ldm r51,%0\nlde %cq,r51\n"		2
reg8: CVUU8(reg4)				"ldr r13,%0\nlde %cq,r13\n"		2
reg8: CVUU8(INDIRI4(addr56))	"ldm r13,%0\nlde %cq,r13\n"		2
reg8: CVUU8(reg8)				"?ldr %cq,%0\n"					1
reg8: CVUU8(INDIRI8(addr56))	"ldm %cq,%0\n"					1

reg1: CVIU1(reg1)				"?ldr %cb,%0\n"					1
reg1: CVIU1(INDIRI1(addr56))	"ldm %cb,%0\n"					1
reg1: CVIU1(reg2)				"?ldr %cb,%0\n"					1
reg1: CVIU1(INDIRI2(addr56))	"ldm %cb,%0\n"					1
reg1: CVIU1(reg4)				"?ldr %cb,%0\n"					1
reg1: CVIU1(INDIRI4(addr56))	"ldm %cb,%0\n"					1
reg1: CVIU1(reg8)				"?ldr %cb,%0\n"					1
reg1: CVIU1(INDIRI8(addr56))	"ldm %cb,%0\n"					1

reg2: CVIU2(reg1)				"ldr r110,%0\nlde %cw,r110\n"	2
reg2: CVIU2(INDIRI1(addr56))	"ldm r110,%0\nlde %cw,r110\n"	2
reg2: CVIU2(reg2)				"?ldr %cw,%0\n"					1
reg2: CVIU2(INDIRI2(addr56))	"ldm %cw,%0\n"					1
reg2: CVIU2(reg4)				"?ldr %cw,%0\n"					1
reg2: CVIU2(INDIRI4(addr56))	"ldm %cw,%0\n"					1
reg2: CVIU2(reg8)				"?ldr %cw,%0\n"					1
reg2: CVIU2(INDIRI8(addr56))	"ldm %cw,%0\n"					1

reg4: CVIU4(reg1)				"ldr r110,%0\nlde %cd,r110\n"	2
reg4: CVIU4(INDIRI1(addr56))	"ldm r110,%0\nlde %cd,r110\n"	2
reg4: CVIU4(reg2)				"ldr r51,%0\nlde %cd,r51\n"		2
reg4: CVIU4(INDIRI2(addr56))	"ldm r51,%0\nlde %cd,r51\n"		2
reg4: CVIU4(reg4)				"?ldr %cd,%0\n"					1
reg4: CVIU4(INDIRI4(addr56))	"ldm %cd,%0\n"					1
reg4: CVIU4(reg8)				"?ldr %cd,%0\n"					1
reg4: CVIU4(INDIRI8(addr56))	"ldm %cd,%0\n"					1
reg4: CVIU4(LOADI4(reg4))		"?ldr %cd,%0\n"			1

reg8: CVIU8(reg1)				"ldr r110,%0\nlde %cq,r110\n"	2
reg8: CVIU8(INDIRI1(addr56))	"ldm r110,%0\nlde %cq,r110\n"	2
reg8: CVIU8(reg2)				"ldr r51,%0\nlde %cq,r51\n"		2
reg8: CVIU8(INDIRI2(addr56))	"ldm r51,%0\nlde %cq,r51\n"		2
reg8: CVIU8(reg4)				"ldr r13,%0\nlde %cq,r13\n"		2
reg8: CVIU8(INDIRI4(addr56))	"ldm r13,%0\nlde %cq,r13\n"		2
reg8: CVIU8(reg8)				"?ldr %cq,%0\n"					1
reg8: CVIU8(INDIRI8(addr56))	"ldm %cq,%0\n"					1

reg1: CVUI1(reg1)				"?ldr %cb,%0\n"			1
reg1: CVUI1(INDIRI1(addr56))	"ldm %cb,%0\n"			1
reg1: CVUI1(reg2)				"?ldr %cb,%0\n"			1
reg1: CVUI1(INDIRI2(addr56))	"ldm %cb,%0\n"			1
reg1: CVUI1(reg4)				"?ldr %cb,%0\n"			1
reg1: CVUI1(INDIRI4(addr56))	"ldm %cb,%0\n"			1
reg1: CVUI1(reg8)				"?ldr %cb,%0\n"			1
reg1: CVUI1(INDIRI8(addr56))	"ldm %cb,%0\n"			1

reg2: CVUI2(reg1)				"?ldr %cb,%0\n"			1
reg2: CVUI2(INDIRI1(addr56))	"ldm %cb,%0\n"			1
reg2: CVUI2(reg2)				"?ldr %cw,%0\n"			1
reg2: CVUI2(INDIRI2(addr56))	"ldm %cw,%0\n"			1
reg2: CVUI2(reg4)				"?ldr %cw,%0\n"			1
reg2: CVUI2(INDIRI4(addr56))	"ldm %cw,%0\n"			1
reg2: CVUI2(reg8)				"?ldr %cw,%0\n"			1
reg2: CVUI2(INDIRI8(addr56))	"ldm %cw,%0\n"			1

reg4: CVUI4(reg1)				"?ldr %cb,%0\ncbd %c\n"			2
reg4: CVUI4(INDIRI1(addr56))	"ldm %cb,%0\n"			2
reg4: CVUI4(reg2)				"?ldr %cw,%0\ncwd %c\n"			2
reg4: CVUI4(INDIRI2(addr56))	"ldm %cw,%0\n"			2
reg4: CVUI4(reg4)				"?ldr %cd,%0\n"			2
reg4: CVUI4(INDIRI4(addr56))	"ldm %cd,%0\n"			2
reg4: CVUI4(reg8)				"?ldr %cd,%0\n"			2
reg4: CVUI4(INDIRI8(addr56))	"ldm %cd,%0\n"			2
reg4: CVUI4(LOADU4(reg4))		"?ldr %cd,%0\n"			2
reg4: CVUI4(LOADU1(reg1))		"?ldr %cd,%0\ncbw %c\n"			2
reg4: CVUI4(LOADI4(reg4))		"?ldr %cd,%0\n"			2

reg8: CVUI8(reg1)				"?ldr %cb,%0\n"			1
reg8: CVUI8(INDIRI1(addr56))	"ldm %cb,%0\n"			1
reg8: CVUI8(reg2)				"?ldr %cw,%0\n"			1
reg8: CVUI8(INDIRI2(addr56))	"ldm %cw,%0\n"			1
reg8: CVUI8(reg4)				"?ldr %cd,%0\n"			1
reg8: CVUI8(INDIRI4(addr56))	"ldm %cd,%0\n"			1
reg8: CVUI8(reg8)				"?ldr %cq,%0\n"			1
reg8: CVUI8(INDIRI8(addr56))	"ldm %cq,%0\n"			1


stmt: ASGNI1(VREGP,reg1)			"# write register\n"	1
stmt: ASGNI1(lp4, reg1)				"ldm %0b,%1b\n"			1
stmt: ASGNI1(addr56, con)			"ldm %0b,=%1\n"			1
stmt: ASGNI1(addr56, reg1)			"ldm %0b,%1b\n"			1
stmt: ASGNI1(addr56, LOADI1(reg1))	"ldm %0b,%1\n"			1
stmt: ASGNI1(addr56, LOADU1(reg1))	"ldm %0b,%1\n"			1
stmt: ASGNI1(addr56, LOADU1(reg4))	"ldm %0b,%1b\n"			1
stmt: ASGNI1(addr56, LOADU4(reg4))	"ldm %0b,%1b\n"			1
stmt: ASGNI1(indir561,con)			"ldm %0b,=%1\n"			1
stmt: ASGNI1(indir561,reg1)			"ldm %0b,%1b\n"			1
stmt: ASGNI1(indir561,LOADI1(reg1)) "ldm %0b,%1\n"			1
stmt: ASGNI1(indir561,LOADU1(reg1)) "ldm %0b,%1\n"			1
stmt: ASGNI1(indir561,LOADI2(reg2)) "ldm %0b,%1\n"			1
stmt: ASGNI1(indir561,LOADU2(reg2)) "ldm %0b,%1\n"			1
stmt: ASGNI1(indir561,LOADI4(reg4)) "ldm %0b,%1\n"			1
stmt: ASGNI1(indir561,LOADU4(reg4)) "ldm %0b,%1\n"			1
stmt: ASGNI1(indir561,LOADI8(reg8)) "ldm %0b,%1\n"			1
stmt: ASGNI1(indir561,LOADU8(reg8)) "ldm %0b,%1\n"			1

stmt: ASGNU1(VREGP,reg1)			"# write register\n"	1
stmt: ASGNU1(lp4, reg1)				"ldm %0b,%1b\n"			1
stmt: ASGNU1(addr56, con)			"ldm %0b,=%1\n"			1
stmt: ASGNU1(addr56, reg1)			"ldm %0b,%1b\n"			1
stmt: ASGNU1(addr56, LOADI1(reg1))	"ldm %0b,%1\n"			1
stmt: ASGNU1(addr56, LOADU1(reg1))	"ldm %0b,%1\n"			1
stmt: ASGNU1(addr56, LOADU1(reg4))	"ldm %0b,%1b\n"			1
stmt: ASGNU1(addr56, LOADU4(reg4))	"ldm %0b,%1b\n"			1
stmt: ASGNU1(indir561,con)			"ldm %0b,=%1\n"			1
stmt: ASGNU1(indir561,reg1)			"ldm %0b,%1b\n"			1
stmt: ASGNU1(indir561,LOADI1(reg1)) "ldm %0b,%1\n"			1
stmt: ASGNU1(indir561,LOADU1(reg1)) "ldm %0b,%1\n"			1
stmt: ASGNU1(indir561,LOADI2(reg2)) "ldm %0b,%1\n"			1
stmt: ASGNU1(indir561,LOADU2(reg2)) "ldm %0b,%1\n"			1
stmt: ASGNU1(indir561,LOADI4(reg4)) "ldm %0b,%1\n"			1
stmt: ASGNU1(indir561,LOADU4(reg4)) "ldm %0b,%1\n"			1
stmt: ASGNU1(indir561,LOADI8(reg8)) "ldm %0b,%1\n"			1
stmt: ASGNU1(indir561,LOADU8(reg8)) "ldm %0b,%1\n"			1

stmt: ASGNI2(VREGP,reg2)			"# write register\n"	1
stmt: ASGNI2(lp4, reg2)				"ldm %0w,%1w\n"			1
stmt: ASGNI2(addr56, con)			"ldm %0w,=%1\n"			1
stmt: ASGNI2(addr56, reg2)			"ldm %0w,%1w\n"			1
stmt: ASGNI2(addr56, LOADU2(reg4))	"ldm %0w,%1w\n"			1
stmt: ASGNI2(addr56, LOADU4(reg4))	"ldm %0w,%1w\n"			1
stmt: ASGNI2(indir562,con)			"ldm %0w,=%1\n"			1
stmt: ASGNI2(indir562,reg2)			"ldm %0w,%1w\n"			1
stmt: ASGNI2(indir562,LOADI2(reg2)) "ldm %0w,%1\n"			1
stmt: ASGNI2(indir562,LOADU2(reg2)) "ldm %0w,%1\n"			1
stmt: ASGNI2(indir562,LOADI4(reg4)) "ldm %0w,%1\n"			1
stmt: ASGNI2(indir562,LOADU4(reg4)) "ldm %0w,%1\n"			1
stmt: ASGNI2(indir562,LOADI8(reg8)) "ldm %0w,%1\n"			1
stmt: ASGNI2(indir562,LOADU8(reg8)) "ldm %0w,%1\n"			1

stmt: ASGNU2(VREGP,reg2)			"# write register\n"	1
stmt: ASGNU2(lp4, reg2)				"ldm %0w,%1w\n"			1
stmt: ASGNU2(addr56, con)			"ldm %0w,=%1\n"			1
stmt: ASGNU2(addr56, reg2)			"ldm %0w,%1w\n"			1
stmt: ASGNU2(addr56, LOADU2(reg4))	"ldm %0w,%1w\n"			1
stmt: ASGNU2(addr56, LOADU4(reg4))	"ldm %0w,%1w\n"			1
stmt: ASGNU2(indir562,con)			"ldm %0w,=%1\n"			1
stmt: ASGNU2(indir562,reg2)			"ldm %0w,%1w\n"			1
stmt: ASGNU2(indir562,LOADI2(reg2)) "ldm %0w,%1\n"			1
stmt: ASGNU2(indir562,LOADU2(reg2)) "ldm %0w,%1\n"			1
stmt: ASGNU2(indir562,LOADI4(reg4)) "ldm %0w,%1\n"			1
stmt: ASGNU2(indir562,LOADU4(reg4)) "ldm %0w,%1\n"			1
stmt: ASGNU2(indir562,LOADI8(reg8)) "ldm %0w,%1\n"			1
stmt: ASGNU2(indir562,LOADU8(reg8)) "ldm %0w,%1\n"			1

stmt: ASGNI4(VREGP,reg4)			"# write register\n"	1
stmt: ASGNI4(addr56,con)			"ldm %0d,=%1\n"			1
stmt: ASGNI4(addr56,reg4)			"ldm %0d,%1d\n"			1
stmt: ASGNI4(addr56,LOADI4(reg4))	"ldm %0d,%1d\n"			1
stmt: ASGNI4(addr56,LOADU4(reg4))	"ldm %0d,%1d\n"			1
stmt: ASGNI4(addr56,LOADI4(reg8))	"ldm %0d,%1d\n"			1
stmt: ASGNI4(addr56,LOADU4(reg8))	"ldm %0d,%1d\n"			1
stmt: ASGNI4(indir564,con)			"ldm %0d,=%1\n"			1
stmt: ASGNI4(indir564,reg4)			"ldm %0d,%1d\n"			1
stmt: ASGNI4(indir564,LOADI4(reg4)) "ldm %0d,%1\n"			1
stmt: ASGNI4(indir564,LOADU4(reg4)) "ldm %0d,%1\n"			1
stmt: ASGNI4(indir564,LOADI4(reg8)) "ldm %0d,%1\n"			1
stmt: ASGNI4(indir564,LOADU4(reg8)) "ldm %0d,%1\n"			1
stmt: ASGNI4(indir564,LOADI8(reg8)) "ldm %0d,%1\n"			1
stmt: ASGNI4(indir564,LOADU8(reg8)) "ldm %0d,%1\n"			1

stmt: ASGNU4(VREGP,reg4)			"# write register\n"	1
stmt: ASGNU4(addr56,con)			"ldm %0d,=%1\n"			1
stmt: ASGNU4(addr56,reg4)			"ldm %0d,%1d\n"			1
stmt: ASGNU4(addr56, LOADI4(reg4))	"ldm %0d,%1d\n"			1
stmt: ASGNU4(addr56, LOADU4(reg4))	"ldm %0d,%1d\n"			1
stmt: ASGNU4(indir564,con)			"ldm %0d,=%1\n"			1
stmt: ASGNU4(indir564,reg4)			"ldm %0d,%1d\n"			1
stmt: ASGNU4(indir564,LOADI4(reg4)) "ldm %0d,%1\n"			1
stmt: ASGNU4(indir564,LOADI4(reg8)) "ldm %0d,%1\n"			1
stmt: ASGNU4(indir564,LOADU4(reg4)) "ldm %0d,%1\n"			1
stmt: ASGNU4(indir564,LOADU4(reg8)) "ldm %0d,%1\n"			1
stmt: ASGNU4(indir564,LOADI8(reg8)) "ldm %0d,%1\n"			1
stmt: ASGNU4(indir564,LOADU8(reg8)) "ldm %0d,%1\n"			1

stmt: ASGNP4(VREGP,reg4)			"# write register\n"	1
stmt: ASGNP4(addr56,con)			"ldm %0d,=%1\n"			1
stmt: ASGNP4(addr56,reg4)			"ldm %0d,%1d\n"			1

stmt: ASGNF4(addr56, freg)				"ldm %0d,r16\n"			1
stmt: ASGNF4(addr56, memf)				"ldm r13,%1\nldm %0,r13\n"  2
stmt: ASGNF4(addr56,reg4)				"ldm %0d,%1d\n" 1
stmt: ASGNF4(addr56,con)				"ldm %0d,%1\n" 1
stmt: ASGNF4(addr56,dreg)				"po8 r52\nd2f r52\nldm %0d,r52\n"  3
stmt: ASGNF4(addr56,CVFF4(dreg))		"po8 r52\nd2f r52\nldm %0d,r52\n"  3
stmt: ASGNF4(addr56, ADDF4(memf, memf))	"ldm r52d,%1\nf2d r52\npu8 r52q\nldm r52d,%2\nf2d r52\npu8 r52q\nad8\npo8 r52\nd2f r52\nldm %0d,r52\n" 10
stmt: ASGNF4(addr56, SUBF4(memf, memf))	"ldm r52d,%1\nf2d r52\npu8 r52q\nldm r52d,%2\nf2d r52\npu8 r52q\nsu8\npo8 r52\nd2f r52\nldm %0d,r52\n" 10
stmt: ASGNF4(addr56, MULF4(memf, memf))	"ldm r52d,%1\nf2d r52\npu8 r52q\nldm r52d,%2\nf2d r52\npu8 r52q\nmu8\npo8 r52\nd2f r52\nldm %0d,r52\n" 10
stmt: ASGNF4(addr56, DIVF4(memf, memf))	"ldm r52d,%1\nf2d r52\npu8 r52q\nldm r52d,%2\nf2d r52\npu8 r52q\ndi8\npo8 r52\nd2f r52\nldm %0d,r52\n" 10

stmt: ASGNI8(VREGP,reg8)  "# write register\n" 1
stmt: ASGNI8(addr56,con)			"ldm %0q,=%1\n"			1
stmt: ASGNI8(addr56,reg8)			"ldm %0q,%1q\n"			1
stmt: ASGNI8(addr56,LOADI8(reg4))	"ldm %0q,%1q\n"			1
stmt: ASGNI8(addr56,LOADU8(reg4))	"ldm %0q,%1q\n"			1
stmt: ASGNI8(indir568,con)			"ldm %0q,=%1\n"			1
stmt: ASGNI8(indir568,reg8)			"ldm %0q,%1q\n"			1
stmt: ASGNI8(indir568,LOADI8(reg8)) "ldm %0q,%1q\n"			1
stmt: ASGNI8(indir568,LOADU8(reg8)) "ldm %0q,%1q\n"			1

stmt: ASGNU8(VREGP,reg8)			"# write register\n"	1
stmt: ASGNU8(addr56,con)			"ldm %0q,=%1\n"			1
stmt: ASGNI8(addr56,reg8)			"ldm %0q,%1q\n"			1
stmt: ASGNU8(addr56,reg8)			"ldm %0q,%1q\n"			1
stmt: ASGNU8(addr56,LOADI8(reg8))	"ldm %0q,%1\n"			1
stmt: ASGNU8(addr56,LOADU8(reg8))	"ldm %0q,%1\n"			1

stmt: ASGNF8(VREGP,reg8)  "# write register\n" 1
stmt: ASGNF8(addr56, memd)			"ldm r52q,%1\nldm %0,r52q\n"  2
stmt: ASGNF8(addr56,dreg)			"po8 %0q\n"  1
stmt: ASGNF8(addr56,CVFF8(dreg))	"po8 %0q\n"  1
stmt: ASGNF8(addr56, ADDF8(memd, memd))	"pu8 %1q\npu8 %2q\nad8\npo8 %0d\n" 4
stmt: ASGNF8(addr56, SUBF8(memd, memd))	"pu8 %1q\npu8 %2q\nsu8\npo8 %0d\n" 4
stmt: ASGNF8(addr56, MULF8(memd, memd))	"pu8 %1q\npu8 %2q\nmu8\npo8 %0d\n" 4
stmt: ASGNF8(addr56, DIVF8(memd, memd))	"pu8 %1q\npu8 %2q\ndi8\npo8 %0d\n" 4

stmt: ASGNB(reg4,INDIRB(reg4))		"ldi r13,=%a\nSysMemcpy00\n"	2

stmt: ARGI2(indir562)		"push %0\n"		1
stmt: ARGI2(reg2)			"pusc %0w\n"	1
stmt: ARGI2(con)			"puw =%0\n"		1
stmt: ARGU2(indir562)		"push %0\n"		1
stmt: ARGU2(reg2)			"pusc %0\n"		1
stmt: ARGU2(con)			"puw =%0\n"		1

stmt: ARGI4(indir564)		"push %0d\n"	1
stmt: ARGI4(reg4)			"pusc %0d\n"	1
stmt: ARGI4(con)			"pud =%0\n"		1
stmt: ARGU4(indir564)		"push %0d\n"	1
stmt: ARGU4(reg4)			"pusc %0d\n"	1
stmt: ARGU4(con)			"pud =%0\n"		1

stmt: ARGP4(indir564)		"push %0d\n"	1
stmt: ARGP4(addr56)			"pea  %0\n"		1
stmt: ARGP4(reg4)			"pusc %0d\n"	1
stmt: ARGP4(con)			"pud =%0\n"		1

stmt: ARGI8(indir568)		"push %0q\n"	1
stmt: ARGI8(reg8)			"pusc %0q\n"	1
stmt: ARGI8(con)			"puq =%0\n"		1
stmt: ARGU8(indir568)		"push %0q\n"	1
stmt: ARGU8(reg8)			"pusc %0q\n"	1
stmt: ARGU8(con)			"puq =%0\n"		1

reg4: CALLI2(addrj)			"jsrc =%0\nadd r7,=%a\n"	2
reg4: CALLU2(addrj)			"jsrc =%0\nadd r7,=%a\n"	2
reg4: CALLI4(addrj)			"jsrc =%0\nadd r7,=%a\n"	2
reg4: CALLI4(reg4)			"jsrc %0d,=0\nadd r7,=%a\n"	2
reg4: CALLU4(addrj)			"jsrc =%0\nadd r7,=%a\n"	2
reg4: CALLU4(reg4)			"jsrc %0d,=0\nadd r7,=%a\n"	2

reg4: CALLI4(INDIRP4(ADDP4(reg4, mem56)))	"ldr r52d,%0\nadd r52d,=%1\njsrc r52d,=0\nadd r7,=%a\n" 4
reg4: CALLU4(INDIRP4(ADDP4(reg4, mem56)))	"ldr r52d,%0\nadd r52d,=%1\njsrc r52d,=0\nadd r7,=%a\n" 4

reg4: CALLI4(addrj)			"jsrc =%0\nadd r7,=%a\n"	2
reg4: CALLU4(addrj)			"jsrc =%0\nadd r7,=%a\n"	2
reg4: CALLP4(addrj)			"jsrc =%0\nadd r7,=%a\n"	2
stmt: CALLV(addrj)			"jsrc =%0\nadd r7,=%a\n"	2
reg4: CALLF4(addrj)			"jsrc =%0\nadd r7,=%a\n"	2
stmt: CALLF4(addrj)			"jsrc =%0\nadd r7,=%a\n"	2
dreg: CALLF8(addrj)			"jsrc =%0\nadd r7,=%a\n"    2
stmt: CALLF8(addrj)			"jsrc =%0\nadd r7,=%a\n"	2

reg4: CALLI2(indir564)		"ldm r52d,%0\njsrc r52d,=0\nadd r7,=%a\n"	3
reg4: CALLU2(indir564)		"ldm r52d,%0\njsrc r52d,=0\nadd r7,=%a\n"	3
reg4: CALLI4(indir564)		"ldm r52d,%0\njsrc r52d,=0\nadd r7,=%a\n"	3
reg4: CALLU4(indir564)		"ldm r52d,%0\njsrc r52d,=0\nadd r7,=%a\n"	3
reg4: CALLP4(indir564)		"ldm r52d,%0\njsrc r52d,=0\nadd r7,=%a\n"	3
stmt: CALLV(indir564)		"ldm r52d,%0\njsrc r52d,=0\nadd r7,=%a\n"	3
reg4: CALLF4(indir564)		"ldm r52d,%0\njsrc r52d,=0\nadd r7,=%a\n"	3
stmt: CALLF4(indir564)		"ldm r52d,%0\njsrc r52d,=0\nadd r7,=%a\n"	3
dreg: CALLF8(indir564)		"ldm r52d,%0\njsrc r52d,=0\nadd r7,=%a\n"	3
stmt: CALLF8(indir564)		"ldm r52d,%0\njsrc r52d,=0\nadd r7,=%a\n"	3

reg4: RETI4(reg4)			"?ldr %cd,%0\n"	1
reg4: RETU4(reg4)			"?ldr %cd,%0\n"	1

stmt: RETI4(reg4)			"# ret\n"
stmt: RETU4(reg4)			"# ret\n"
stmt: RETI8(reg8)			"# ret\n"
stmt: RETU8(reg8)			"# ret\n"
stmt: RETF4(INDIRF4(addr56))	"ldm r16d,%0\n" 1
stmt: RETF4(dreg)			"# ret\n"
stmt: RETF8(dreg)			"# ret\n"
stmt: RETP4(reg4)			"# ret\n"

stmt: JUMPV(addrj)				"jmp =%0\n"	1
stmt: JUMPV(INDIRP4(addr56))	"jmp %0\n"	1
stmt: LABELV					"%a:\n"

stmt: ARGB(INDIRB(reg4))	"# ARGB\n"

addrj: ADDRGP4				"%a"

reg4: ASMRETU4					"#\n" 1
reg8: ASMRETU8					"#\n" 1
stmt: ASMRETU4					"#\n" 1
stmt: ASMRETU8					"#\n" 1

memf:	INDIRF4(addr56)		"%0"
memd:	INDIRF8(addr56)		"%0q"

dreg: memf					"ldm r52d,%0\nf2d r52\npu8 r52q\n" 3
dreg: INDIRF8(addr56)		"pu8 %0q\n" 1
dreg: memd					"ldm r52q,%0\npu8 r52q\n" 2
dreg: mem5					"pu8 %0q\n" 1
dreg: reg8					"pu8 %aq\n" 1
dreg: CNSTF4				"ldm r52d,%0\nf2d r52\npu8 r52q\n" 3
dreg: CNSTF8				"pu8 =%0q\n" 1
dreg: CVIF4(reg4)			"ldr r52d,%0\ncdq r52d\ni2d r52\n" 3

dreg: CVFF4(dreg)			"# do nothing\n"
dreg: CVFF8(dreg)			"# do nothing\n"
reg2: CVFI2(dreg)			"d2i r52\nldr %cw,r52\n"		2
reg4: CVFI4(dreg)			"d2i r52\nldr %cd,r52d\n"		2
reg8: CVFI8(dreg)			"d2i %cq\n"						1
dreg: CVIF8(reg1)			"ldr r52b,%0\ncbq r52d\ni2d r52\n"	4
dreg: CVIF8(reg2)			"ldr r52w,%0\ncwq r52d\ni2d r52\n"	4
dreg: CVIF8(reg4)			"ldr r52d,%0\ncdq r52d\ni2d r52\n"	4
dreg: CVIF8(reg8)			"i2d %0\n"				2

caddr: ADDRGP4				"%a"

freg: CALLF4(caddr)			"jsrc =%0\nadd r7,=%a\n" 2

stmt: ARGF4(INDIRF4(addr56))	"push %0d\n" 1
stmt: ARGF4(dreg)				"po8 r52\nd2f r52\npusc r52d\n" 3
stmt: ARGF8(dreg)				"po8 r52\npusc r52q\n" 2

stmt: ARGF8(INDIRF8(addr56))	"push %0q\n" 1

dreg: NEGF4(dreg)			"ne8\n"	1
dreg: NEGF8(dreg)			"ne8\n"	1

dreg: ADDF4(dreg,dreg)		"ad8\n"	1
dreg: SUBF4(dreg,dreg)		"su8\n"	1
dreg: MULF4(dreg,dreg)		"mu8\n"	1
dreg: DIVF4(dreg,dreg)		"di8\n"	1

dreg: ADDF8(dreg,dreg)		"ad8\n"	1
dreg: SUBF8(dreg,dreg)		"su8\n"	1
dreg: MULF8(dreg,dreg)		"mu8\n"	1
dreg: DIVF8(dreg,dreg)		"di8\n"	1

stmt: EQF4(dreg,dreg)			"cm8\nlje  =%a\n"	2
stmt: GEF4(dreg,dreg)			"cm8\nljge =%a\n"	2
stmt: GTF4(dreg,dreg)			"cm8\nljg  =%a\n"	2
stmt: LEF4(dreg,dreg)			"cm8\nljle =%a\n"	2
stmt: LTF4(dreg,dreg)			"cm8\nljl  =%a\n"	2
stmt: NEF4(dreg,dreg)			"cm8\nljne =%a\n"	2

stmt: EQF8(dreg,dreg)			"cm8\nlje  =%a\n"	2
stmt: GEF8(dreg,dreg)			"cm8\nljge =%a\n"	2
stmt: GTF8(dreg,dreg)			"cm8\nljg  =%a\n"	2
stmt: LEF8(dreg,dreg)			"cm8\nljle =%a\n"	2
stmt: LTF8(dreg,dreg)			"cm8\nljl  =%a\n"	2
stmt: NEF8(dreg,dreg)			"cm8\nljne =%a\n"	2

%%
static void progbeg(long argc, char *argv[]) {

	long		i;
	
	{
		union {
			char c;
			long i;
		} u;
		u.i = 0;
		u.c = 1;
		swap = (long)((unsigned)(u.i == 1)) != IR->little_endian;
	}
	parseflags(argc, argv);

	longreg[R14] = mkreg("r14", R14, 1, IREG);	// 0
	longreg[R15] = mkreg("r15", R15, 1, IREG);
	longreg[R16] = mkreg("r16", R16, 1, IREG);
	longreg[R22] = mkreg("r22", R22, 1, IREG);
	longreg[R23] = mkreg("r23", R23, 1, IREG);
	longreg[R24] = mkreg("r24", R24, 1, IREG);
	longreg[R25] = mkreg("r25", R25, 1, IREG);
	longreg[R26] = mkreg("r26", R26, 1, IREG);
	longreg[R27] = mkreg("r27", R27, 1, IREG);
	longreg[R28] = mkreg("r28", R28, 1, IREG);
	longreg[R29] = mkreg("r29", R29, 1, IREG);

	longlongreg[R56] = mkreg("r56", R56, 1, IREG);	// 11
	longlongreg[R60] = mkreg("r60", R60, 1, IREG);
	longlongreg[R64] = mkreg("r64", R64, 1, IREG);
	longlongreg[R68] = mkreg("r68", R68, 1, IREG);
	
	shortreg[R72] = mkreg("r72", R72, 1, IREG);	// 15
	shortreg[R73] = mkreg("r73", R73, 1, IREG);
	shortreg[R74] = mkreg("r74", R74, 1, IREG);
	shortreg[R75] = mkreg("r75", R75, 1, IREG);
	shortreg[R76] = mkreg("r76", R76, 1, IREG);
	shortreg[R77] = mkreg("r77", R77, 1, IREG);
	shortreg[R78] = mkreg("r78", R78, 1, IREG);
	shortreg[R79] = mkreg("r79", R79, 1, IREG);

	charreg[R80] = mkreg("r80", R80, 1, IREG);	// 23
	charreg[R81] = mkreg("r81", R81, 1, IREG);
	charreg[R82] = mkreg("r82", R82, 1, IREG);
	charreg[R83] = mkreg("r83", R83, 1, IREG);
	charreg[R84] = mkreg("r84", R84, 1, IREG);
	charreg[R85] = mkreg("r85", R85, 1, IREG);
	charreg[R86] = mkreg("r86", R86, 1, IREG);
	charreg[R87] = mkreg("r87", R87, 1, IREG);

	for(i=0; i<8; i++) fltreg[i] = mkreg("%d", i, 0, FREG);
//	fltreg[R88] = mkreg("R88", R88, 1, FREG);
//	fltreg[R96] = mkreg("R96", R96, 1, FREG);

	charregw = mkwildcard(charreg);
	shortregw = mkwildcard(shortreg);
	longregw = mkwildcard(longreg);
	longlongregw = mkwildcard(longlongreg);
	fltregw = mkwildcard(fltreg);

	tmask[IREG] =  (1<<R16) | (1<<R22) | (1<<R23) | (1<<R24) | (1<<R25) | (1<<R26)
				 | (1<<R27) | (1<<R28) | (1<<R29) | (1<<R56) | (1<<R60) | (1<<R64) | (1<<R68) | (1<<R72)
				 | (1<<R73) | (1<<R74) | (1<<R75) | (1<<R76) | (1<<R77) | (1<<R78) | (1<<R79) | (1<<R80)
				 | (1<<R81) | (1<<R82) | (1<<R83) | (1<<R84) | (1<<R85) | (1<<R86) | (1<<R87);
	vmask[IREG] = 0;

	tmask[FREG] = 0xff;
	vmask[FREG] = 0;

	cseg = exchkstk = 0;

	asmresult = mkreg("r16", R16, 1, IREG);
}

static Symbol rmap(long opk) {

	switch (optype(opk)) {
	 case B:	// struct
	 case P:	// pointer
		return longregw;
	 case I:
	 case U:
		if( opsize(opk)==1 ) return charregw;
		else if( opsize(opk)==2 ) return shortregw;
		else if( opsize(opk)==4 ) return longregw;
		else return longlongregw;
	 case F:
		return fltregw;
	 default:
		return 0;
	}
}

static void segment(long n) {

	if( (cseg==CODE || cseg==LIT || cseg==DATA) && (n==0 || n==BSS) ) print("@code_end\n");
	if( cseg==BSS && n!=BSS ) print("@zerodata_end\n");
	if( (n==CODE || n==LIT || n==DATA) && (cseg==0 || cseg==BSS) ) print("@code\n");
	if( n==BSS && cseg!=BSS ) print("@zerodata\n");

	cseg = n;
}

static void progend(void) {
	segment(0);
	if( exchkstk ) print("extrn __chkstk:BYTE\n");
	print("@end\n");
}

static void target(Node p) {

        assert(p);
        switch (specific(p->op)) {
		 case ASMRET+U:
		 case ASM0+U:
		 case ASM1+U:
		 case ASM2+U:
			setreg(p, asmresult);
			break;
         case ASGN+B:
                rtarget(p, 0, longreg[R15]);
                rtarget(p->kids[1], 0, longreg[R14]);
                break;
         case ARG+B:
                rtarget(p->kids[0], 0, longreg[R14]);
                break;
         case CVF+I:
                setreg(p, longreg[R16]);
                break;
         case CALL+I:
		 case CALL+U:
		 case CALL+P:
		 case CALL+V:
                setreg(p, longreg[R16]);
                break;
         case RET+I:
		 case RET+U:
		 case RET+P:
                rtarget(p, 0, longreg[R16]);
                break;
/*		 case RET+F:
                rtarget(p, 0, longreg[R16]);
                break;
*/
        }
}

//****************************************
static void clobber(Node p) {

        static long nstack = 0;

        assert(p);
        nstack = ckstack(p, nstack);
        switch (specific(p->op)) {
/*
        case RSH+I:
		case RSH+U:
		case LSH+I:
		case LSH+U:
                if (generic(p->kids[1]->op) != CNST
                && !(   generic(p->kids[1]->op) == INDIR
                     && specific(p->kids[1]->kids[0]->op) == VREG+P
                     && p->kids[1]->syms[RX]->u.t.cse
                     && generic(p->kids[1]->syms[RX]->u.t.cse->op) == CNST
)) {
                        spill(1<<R13, 1, p);
                }
                break;
*/
        case ASGN+B:
		case ARG+B:
                spill(~tmask[IREG] & ((1<<R14) | (1<<R15)), IREG, p);
                break;
        case EQ+F:
		case LE+F:
		case GE+F:
		case LT+F:
		case GT+F:
		case NE+F:
                spill(~tmask[IREG] & (1<<R16), IREG, p);
                if( specific(p->op) == EQ+F ) p->syms[1] = findlabel(genlabel(1));
                break;

// GEEKDO: WHAT REGISTERS DO WE *REALLY* NEED TO spill() BELOW?????

        case CALL+F:
//				spill(tmask[IREG] & ~((1<<R14) | (1<<R15) | (1<<R16) | (1<<17) | (1<<18) | (1<<19) | (1<<20) | (1<<21)), IREG, p);
//				spill(~tmask[IREG] & ((1<<R14) | (1<<R15) | (1<<R16)), IREG, p);
//               break;
        case CALL+I:
		case CALL+U:
		case CALL+P:
		case CALL+V:
//				spill(~tmask[IREG] & ((1<<R14) | (1<<R15) | (1<<R16)), IREG, p);
//				spill(usedmask[IREG] & ~(1<<R16), IREG, p);
                break;
        }
}


#define isfp(p) (optype((p)->op)==F)

long ckstack(Node p, long n) {
        long i;

        for (i = 0; i < NELEMS(p->x.kids) && p->x.kids[i]; i++)
                if (isfp(p->x.kids[i]))
                        n--;
        if (isfp(p) && p->count > 0)
                n++;
        if (n > 8)
                error("expression too complicated\n");
//        debug(fprint(errfp, "(ckstack(%x)=%d)\n", p, n));
        assert(n >= 0);
        return n;
}
long sametree(Node p, Node q) {
        return (p == NULL && q == NULL)
        || (p && q && p->op == q->op && p->syms[0] == q->syms[0]
                && sametree(p->kids[0], q->kids[0])
                && sametree(p->kids[1], q->kids[1]));
}

long memop56(Node p, long incdec) {

	assert(p);
	assert(generic(p->op) == ASGN);
	assert(p->kids[0]);
	assert(p->kids[1]);
	if( generic(p->kids[1]->kids[0]->op) == INDIR
	 && sametree(p->kids[0], p->kids[1]->kids[0]->kids[0]) ) return 3+incdec;
	else return LBURG_MAX;
}

//*******************************************
static void emit2(Node p) {

	long op = specific(p->op);

//stmt: ASGNU4(addr56,  BORU4(indir56,rc))	"# or %0d,%2\n" 1
//	fprint(stderr, "op=%d specific(p->op)=%d\n", op, specific(p->op));

#define preg(f) ((f)[getregnum(p->x.kids[0])]->x.name)

//print("EMIT2 op=%d p->op=%d\r\n", op, p->op);

/*
	if( op==ASGN+I ) {
		// ASGNI4(ADDRFP4(w), BORI4(INDIRI4(ADDRFP4(w)), CNSTI4(2)))ldm [r6][8]d,r29d
		long	s;
		s = opsize(p->op);
		print("\n");
		dumptree(p);
		print("\n");
		dumptree(p->kids[0]);
		print("\n");
		dumptree(p->kids[1]);
		print("\n");
		dumptree(p->kids[1]->kids[0]);
		print("\n");
		dumptree(p->kids[1]->kids[1]);
		print("\n");

		print("ldm %0,%1\nor %0,=%2\nldm %1,%0\n");
	} else
*/
/*
// convert CHAR to LONG
	if (op == CVI+I && opsize(p->op) == 4 && opsize(p->x.kids[0]->op) == 1) {
		print("ldr %sb,%s\ncbd %s\n", p->syms[RX]->x.name, preg(charreg), p->syms[RX]->x.name);
// convert CHAR to DWORD
	} else if (op == CVI+U && opsize(p->op) == 4 && opsize(p->x.kids[0]->op) == 1) {
		print("ldr %sb,%s\ncbd %s\n", p->syms[RX]->x.name, preg(charreg), p->syms[RX]->x.name);
// convert SHORT to LONG
	} else if (op == CVI+I && opsize(p->op) == 4 && opsize(p->x.kids[0]->op) == 2) {
		print("ldr %sw,%s\ncwd %s\n", p->syms[RX]->x.name, preg(shortreg), p->syms[RX]->x.name);
// convert SHORT to DWORD
	} else if (op == CVI+U && opsize(p->op) == 4 && opsize(p->x.kids[0]->op) == 2) {
		print("ldr %sw,%s\ncwd %s\n", p->syms[RX]->x.name, preg(shortreg), p->syms[RX]->x.name);
// convert BYTE to LONG
	} else if (op == CVU+I && opsize(p->op) == 4 && opsize(p->x.kids[0]->op) == 1) {
		print("lde %s,%s\n", p->syms[RX]->x.name, preg(charreg));
// convert BYTE to DWORD
	} else if (op == CVU+U && opsize(p->op) == 4 && opsize(p->x.kids[0]->op) == 1) {
		print("lde %s,%s\n", p->syms[RX]->x.name, preg(charreg));
// convert WORD to LONG
	} else if (op == CVU+I && opsize(p->op) == 4 && opsize(p->x.kids[0]->op) == 2) {
		print("lde %s,%s\n", p->syms[RX]->x.name, preg(shortreg));
// convert WORD to DWORD
	} else if (op == CVU+U && opsize(p->op) == 4 && opsize(p->x.kids[0]->op) == 2) {
		print("lde %s,%s\n", p->syms[RX]->x.name, preg(shortreg));
	} else if( generic(op) == CVF ) {
*/
/*	} else if( op == CVI+F ) {
		if( opsize(p->op) == 4 ) print("ldm r52,%0\ncdq r52\ni2d r52\n", preg(longreg));
*/
//	} else
		if (generic(op) == CVI || generic(op) == CVU || generic(op) == LOAD) {

		char	*dst;
		char	*src;
		long	srcopsize=opsize(p->x.kids[0]->op),	dstopsize=opsize(p->op);
		long	srcreg=getregnum(p->x.kids[0]),		dstreg=getregnum(p);

		
		if( dstopsize==1 )		dst = charreg[dstreg]->x.name;
		else if( dstopsize==2 )	dst = shortreg[dstreg]->x.name;
		else if( dstopsize==4 )	dst = longreg[dstreg]->x.name;
		else					dst = longlongreg[dstreg]->x.name;

		if( srcopsize==1 ) 		src = charreg[srcreg]->x.name;
		else if( srcopsize==2 )	src = shortreg[srcreg]->x.name;
		else if( srcopsize==4 )	src = longreg[srcreg]->x.name;
		else					src = longlongreg[srcreg]->x.name;

		assert(dstopsize <= srcopsize);
		if (dst != src) {
			if( dstopsize==1 ) print("ldr %sb,%sb\n", dst, src);
			else if( dstopsize==2 ) print("ldr %sw,%sw\n", dst, src);
			else if( dstopsize==4 ) print("ldr %sd,%sd\n", dst, src);
			else if( dstopsize==8 ) print("ldr %sq,%sq\n", dst, src);
		}
	} else if(op == ARG+B) {
//		print("sub esp,%d\nmov edi,esp\nmov ecx,%d\nrep movsb\n", roundup(p->syms[0]->u.c.v.ill, 4), p->syms[0]->u.c.v.ill);
		print("sub r7,=%d\nldr r15,r7\nldi r13,=%d\nSysMemcpy00 ;EMIT2-ARG+B\n", (long)roundup(p->syms[0]->u.c.v.ill, 4), (long)p->syms[0]->u.c.v.ill);
	} else if( generic(op)==ASM0 ) {
		print("%s", p->syms[0]->name);
	} else if( generic(op)==ASMRET || generic(op)==ASM2 ) {
//		print("%s", p->syms[0]->name);
		if( p->kids[0] ) emit2(p->kids[0]);
		if( p->kids[1] ) emit2(p->kids[1]);
	}
}

// mkactual is in gen.c, and simply makes sure the arg stack pointer is
// aligned to the size indicated by mkactual()'s first argument (DWORD,
// in this case), and then moves the arg stack pointer by the size
// of the argument (indicated by mkactual()'s second argument).
// i.e., doarg() simply allocates the space on the stack for the argument 'p'.
static void doarg(Node p) {

	assert(p && p->syms[0]);
	mkactual(4, p->syms[0]->u.c.v.ill);
}

static void blkfetch(long k, long off, long reg, long tmp) {}

static void blkstore(long k, long off, long reg, long tmp) {}

static void blkloop(long dreg, long doff, long sreg, long soff, long size, long tmps[]) {}

static void local(Symbol p) {

	if (isfloat(p->type)) p->sclass = AUTO;
	if (askregvar(p, (*IR->x.rmap)(ttob(p->type))) == 0) {
		assert(p->sclass == AUTO);
		offset = roundup(offset + p->type->size, p->type->align < 4 ? 4 : p->type->align);
		p->x.offset = -offset;
		p->x.name = stringd(-offset);
	}
}

static void function(Symbol f, Symbol caller[], Symbol callee[], long n) {
	long 		i, argsize;
	unsigned	mask;

	usedmask[0] = usedmask[1] = 0;
	freemask[0] = freemask[1] = ~(unsigned)0;
	offset = maxoffset = maxargoffset = 8;	// for retaddr & saved R6

	argsize = 0;
	for(i=0; callee[i]; i++) {
			Symbol p = callee[i];
			Symbol q = caller[i];
			assert(q);
			p->x.offset = q->x.offset = offset;
			p->x.name = q->x.name = stringf("%d", p->x.offset);
			p->sclass = q->sclass = AUTO;
			offset += roundup(q->type->size, 4);
			argsize += q->type->size;
	}
	assert(caller[i]==0);

	offset = maxoffset = 0;
	
	gencode(caller, callee);

	framesize = roundup(maxoffset, 4);

	mask = usedmask[IREG];
	
	print("@align 2\n");
	print("%s:\n", f->x.name);
	print("pusc r6\n");
	print("ldr r6,r7\n");
	if (framesize >= 4096) print("ldi r16,=%d\ncall __chkstk\n", framesize);
	else if( framesize > 0 ) print("sub r7,=%d\n", framesize);
	for(i=0; i<31; i++) {
		if( mask & (1<<i) ) {
			if( i<11 ) {
				if( i!=R16 ) printf("pusc %sd\n", longreg[i]->name);
			} else if( i<15 ) printf("pusc %sq\n", longlongreg[i]->name);
			else if( i<23 ) printf("lde r13,%s\npusc r13\n", shortreg[i]->name);
			else printf("lde r13,%s\npusc r13\n", charreg[i]->name);
		}
	}
			
	emitcode();

	for(i=30; i>=0; i--) {
		if( mask & (1<<i) ) {
			if( i<11 ) {
				if( i!=R16 ) printf("popc %s\n", longreg[i]->name);
			} else if( i<15 ) printf("popc %sq\n", longlongreg[i]->name);
			else if( i<23 ) printf("popc r13\nldr %s,r13w\n", shortreg[i]->name);
			else printf("popc r13\nldr %s,r13b\n", charreg[i]->name);
		}
	}
	print("ldr r7,r6\n");
	print("popc r6\n");
	print("retc\n");
	if (framesize >= 4096) exchkstk = 1;
}

//***********************************
static void defsymbol(Symbol p) {

	if (p->scope >= LOCAL && p->sclass == STATIC) p->x.name = stringf("L%d", genlabel(1));
	else if (p->generated) p->x.name = stringf("L%s", p->name);
	else if (p->scope == GLOBAL || p->sclass == EXTERN) p->x.name = stringf("_%s", p->name);
	else if (p->scope == CONSTANTS && (isint(p->type) || isptr(p->type))
		  && p->name[0] == '0' && p->name[1] == 'x') p->x.name = stringf("0x%s", &p->name[2]);
	else p->x.name = p->name;
}

//***********************************************
static void address(Symbol q, Symbol p, long n) {

	if( p->scope == GLOBAL || p->sclass == STATIC || p->sclass == EXTERN ) {
		q->x.name = stringf("%s%s%d", p->x.name, n >= 0 ? "+" : "", n);
	} else {
		assert(n <= INT_MAX && n >= INT_MIN);
		q->x.offset = p->x.offset + n;
		q->x.name = stringd(q->x.offset);
	}
}

//******************************************************
static void defconst(long suffix, long size, Value v) {

	assert(sizeof(float)==4 && sizeof(double)==8);
		 if (suffix == I && size == 1)	print("BYTE %d\n", (unsigned long)v.ull);
	else if (suffix == I && size == 2)	print("WORD %d\n", (unsigned long)v.ill);
	else if (suffix == I && size == 4)	print("DWORD %d\n",(unsigned long)v.ill);
//	else if (suffix == I && size == 8)	print("QWORD 0x%8.8x%8.8x\n",((unsigned long long)v.ill)>>32, v.ill & 0x0FFFFFFFF);
	else if (suffix == I && size == 8)	print("QWORD 0x%16.16X\n",v.ill);
	else if (suffix == U && size == 1) {
		print("BYTE 0x%x\n", (unsigned)((unsigned char)v.ull));
	} else if (suffix == U && size == 2) {
		print("WORD 0x%x\n", (unsigned)((unsigned short)v.ull));
	} else if (suffix == U && size == 4) {
		print("DWORD 0x%x\n", (unsigned)v.ull);
//	else if (suffix == U && size == 8)	print("QWORD 0x%8.8x%8.8X\n",((unsigned long long)v.ull)>>32, v.ull & 0x0FFFFFFFF);
	} else if (suffix == U && size == 8) {
		print("QWORD 0x%16.16X\n",v.ull);
	} else if (suffix == P && size == 4)	print("DWORD 0x%x\n", (unsigned)v.p);
	else if (suffix == F && size == 4) {
		float f = (float)v.d;
		print("DWORD 0x%x\n", *(unsigned *)&f);
	} else if (suffix == F && size == 8) {
//		double d = (double)v.d;
		print("QWORD 0x%X\n", *(unsigned long long *)&v.d);
//	} else if (suffix == F && size == 8) {
//		double d = v.d;
//		unsigned *p = (unsigned *)&d;
//		print("dd 0%xH\ndd 0%xH\n", p[swap], p[!swap]);
	} else assert(0);
}
static void defaddress(Symbol p) {
        print("DWORD %s\n", p->x.name);
}
static void defstring(long n, char *str) {
        char *s;

        for (s = str; s < str + n; s++)
                print("BYTE %d\n", (*s)&0377);
}
static void export(Symbol p) {
        print("public %s\n", p->x.name);
}
static void import(Symbol p) {
//        long oldseg = cseg;

        if (p->ref > 0) {
//			segment(0);
			print("extern %s:BYTE\n", p->x.name);
//			segment(oldseg);
        }
}
static void global(Symbol p) {

		Type	t;

        print("@align %d\n", p->type->align > 4 ? 4 : p->type->align);
        print("%s:\n", p->x.name);
		for(t=p->type; isarray(t); t=t->type) ;
		if( t->op==STRUCT ) print("@align 1\n");
}
static void space(long n) {
        if( cseg==BSS ) print("BYTE [%d]\n", n);
		else print("BYTE [%d] 0\n", n);
}

Interface kintIR = {
	{1, 1, 0},  /* char */
	{2, 2, 0},  /* short */
	{4, 4, 0},  /* int */
	{4, 4, 0},  /* long */
	{8, 4, 0},  /* long long */
	{4, 4, 1},  /* float */
	{8, 4, 1},  /* double (size, alignment, outofline) */
	{8, 4, 1},  /* long double */
	{4, 4, 0},  /* T * */
	{0, 1, 0},  /* struct */
	1,        /* little_endian */
	0,        /* mulops_calls */
	0,        /* wants_callb */
	1,        /* wants_argb */
	0,        /* left_to_right */
	0,        /* wants_dag */
	0,        /* unsigned_char */
	address,
	blockbeg,
	blockend,
	defaddress,
	defconst,
	defstring,
	defsymbol,
	emit,
	export,
	function,
	gen,
	global,
	import,
	local,
	progbeg,
	progend,
	segment,
	space,
	0, 0, 0, 0, 0, 0, 0,
	{
		1, rmap,
		blkfetch, blkstore, blkloop,
		_label,
		_rule,
		_nts,
		_kids,
		_string,
		_templates,
		_isinstruction,
		_ntname,
		emit2,
		doarg,
		target,
		clobber,
	}
};
